/*
    Gnu-Percolation is a program to simulate 2D percolation.
    Copyright (C) 2001 Raphael Cerf

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    E-mail contact: Raphael.Cerf@math.u-psud.fr
    Ordinary mail: 
	Raphael Cerf, Mathematique, batiment 425
	Universite Paris Sud, 91405 Orsay Cedex

*/

#include <unistd.h>
#include <gtk--/accelgroup.h>
#include <gtk--.h>
#include <gtk--/main.h>
#include <gtk--/button.h>
#include <gtk--/style.h>
#include <gtk--/box.h>
#include <gtk--/drawingarea.h>
#include <gtk--/adjustment.h>
#include <gtk--/scale.h>
#include <gtk--/box.h>
#include <gtk--/frame.h>
#include <gtk--/label.h>
#include <gtk--/spinbutton.h>
#include <gtk--/adjustment.h>
#include <gtk--/window.h>
#include <gtk--/separator.h>
#include <gtk--/scrollbar.h>
#include <gtk--/checkbutton.h>
#include <string>
#include <gtk--/menu.h>
#include <gtk--/menubar.h>
#include <gtk--/optionmenu.h>
#include <gtk--/separator.h>
#include <gtk--/radiobutton.h>
#include <math.h>
#include <pthread.h>

#include <time.h>
#include <limits.h>
#include <vector.h>
#include <iterator.h>
#include <algo.h>
#include <iostream.h>
#include <math.h>
#include <queue>
#include "MersenneTwister.h"

#define NTHREADS 10

enum algo_t {Static, Dynamic};

namespace computation {

enum bc_t {free, wired, toric, Dobrushin};

enum direction_t {left, right, up, down};

struct Site {
    int x;
    int y;
    int z;
};

class Sites {
	MTRand mtrand_hindex;
	MTRand mtrand_vindex;
	MTRand mtrand_dindex;
	MTRand mtrand_flip;
	MTRand mtrand_hindex0;
	MTRand mtrand_vindex0;
	MTRand mtrand_dindex0;
	MTRand mtrand_flip0;
	MTRand mtrand_hindex1;
	MTRand mtrand_vindex1;
	MTRand mtrand_dindex1;
	MTRand mtrand_flip1;
	bool *sites;
	bool *marks;
	float *unif;
	bool *edges;
	int hindex_max, vindex_max, dindex_max;
public:
	bc_t bc;
	int hsize, vsize, fsize, depth;
	float prob, temp;
	int position;
	bool *sites_position;
	float field, epsi;
	Sites();
	~Sites();

	void init(int h, int v, int d, int pos=0);
	void resample(); 
	void set_bc(int num);
	void set_site_to_first(Site *s);
	void random_site(Site *s);
	void increase_site(Site *s);
	inline Site Sites::neighbor_of_site(Site *s, int d);
	bool is_site_first(Site &s) const;
	bool is_site_last(Site &s) const;
	bool is_top_site(Site s) const;
	bool is_bottom_site(Site s) const;
	bool is_boundary_site(Site s) const;
	inline float unif_status(Site s, int d) const;
	inline bool edge_status(Site s, int d) const;
	inline bool mark_status(Site s) const;
	bool site_status(Site s) const;
	bool site_status(int i, int j) const;
void push_open_neighbours_of_site(Site *current_site, queue<Site> *neighbours); 
void push_open_neighbours_of_bulk_site(Site *current_site, queue<Site> *neighbours); 
void push_open_neighbours_of_boundary_site(Site *current_site, queue<Site> *neighbours); 
	int random_hindex(); 
	int random_vindex(); 
	int random_dindex(); 
	int random_hindex0(); 
	int random_vindex0(); 
	int random_dindex0(); 
	int random_hindex1(); 
	int random_vindex1(); 
	int random_dindex1(); 
	void set_prob(float p);
	void set_temp(float T);
	void set_field(float h);
	void set_pos(int pos);
	void set_mark(Site s, bool v);
	void set_site(Site s, bool v);
	void set_unif(Site s, int d, float v);
	void set_edge(Site s, int d, bool v);
};

class Labels : public Sites {
	MTRand mtrand_hindex;
	MTRand mtrand_vindex;
	MTRand mtrand_label;
	int *labels;
	int hindex_max, vindex_max, dindex_max;
	int label_max;
	int *labels_position;
public:
	Labels();
	~Labels();

	void restart();
	void init(int h, int v, int d, int pos=0);
	void set_pos(int pos);
	int label_status(int i, int j) const;
	int label_status(Site s) const;
	int random_hindex(); 
	int random_vindex(); 
	int random_dindex(); 
	int random_label();
	void set_random_label(Site s);
	void set_label(Site s, int v);
	void print();
	void report();
};


class LatticeAlgo : public Labels {
	int delay_report; 
	int seed;
	bool synchronize;
	bool wait;
	int speed;
	int threshold;
	int iteration;
	int iteration0;
	int iteration1;
	MTRand mtrand_algo;
	Site changed_site;
	queue<Site> changed_sites; 
	bool changed_sites_lock; 
	bool waiting;
public:
	LatticeAlgo();
	~LatticeAlgo();
	void lock_changed_sites();
	void unlock_changed_sites();
	bool is_changed_sites_locked();
	void wait_and_lock_changed_sites();
	void init(int h, int v, int d,
float p, float T, int pos=0, int t=0, int sp=0, bool sync=false, bool w=false,
		int d_r=0, int se=4375U);
	int run(algo_t alg, int id=1, int max_iteration=-1);
	int mark_cluster(Site s, queue<Site> *frontier, int l);
	int labelize_cluster(Site s, int l,
		queue<Site> *frontier);
	int labelize_and_grow_cluster(Site s, int l,
		queue<Site> *frontier);
	float update_site_and_labels_with_prob();
	void clear_marks();
	int update_site_and_labels_with_threshold();
	int static0(int id, int max_iteration=-1);
	int growth0(int id, int max_iteration=-1);
	void set_synchronize(bool sync);
	void set_wait(bool w);
	void set_speed(int sp);
	void set_prob(float p);
	void set_pos(int pos);
	void set_threshold(int t);
	int get_iteration();
	void set_temp(float T);
	void set_field(float h);
	bool next_site(int *i, int *j);
	void wait_or_synchronize();
	bool is_changed_sites_empty();
	void empty_changed_sites();
	void set_label_and_push_site(Site s, int v);
};

Sites::Sites() {
	sites=new(bool);
	marks=new(bool);
	unif=new(float);
	edges=new(bool);
}

Sites::~Sites() {
	delete [] sites;	
	delete [] marks;	
	delete [] unif;	
	delete [] edges;	
}

void Sites::set_prob(float p) {
	prob=p;
}

void Sites::set_temp(float T) {
	temp=T;
}

void Sites::set_field(float h) {
	field=h;
}

void Sites::set_pos(int pos) {
	position=pos;
	sites_position=sites+position*fsize; 
}

void Sites::set_bc(int num) {
	switch(num) {
		case 0: bc=free;
		break;
		case 1: bc=wired;
		break;
		case 2: bc=toric;
		break;
		case 3: bc=Dobrushin;
		break;
	}
}


inline void Sites::set_site_to_first(Site *s)
{ s->x=s->y=s->z=0; }

inline void Sites::random_site(Site *s)
{ 
		s->x=random_hindex0();
		s->y=random_vindex0();
		s->z=random_dindex0();
}

inline void Sites::increase_site(Site *s)
{ 
	if (s->z<dindex_max) { s->z++; }
	else {
		s->z=0;
		if (s->y<vindex_max) { s->y++; }
		else {
			s->y=0;
			if (s->x<hindex_max) { s->x++; }
			else {
				s->x=0;
			}
		}
	}
}

inline Site Sites::neighbor_of_site(Site *s, int d)
{ 
	Site neighbor;

	switch (d) {
		case 0:
			if (s->x<hindex_max) { 
				neighbor.x=s->x+1;
			} else {
				neighbor.x=0;
			}
			neighbor.y=s->y;
			neighbor.z=s->z;
		break;
		case 1:
			neighbor.x=s->x;
			if (s->y<vindex_max) { 
				neighbor.y=s->y+1;
			} else {
				neighbor.y=0;
			}
			neighbor.z=s->z;
		break;
		case 2:
			if (s->x>0) { 
				neighbor.x=s->x-1;
			} else {
				neighbor.x=hindex_max;
			}
			neighbor.y=s->y;
			neighbor.z=s->z;
		break;
		case 3:
			neighbor.x=s->x;
			if (s->y>0) { 
				neighbor.y=s->y-1;
			} else {
				neighbor.y=vindex_max;
			}
			neighbor.z=s->z;
		break;
	}
	return(neighbor);
}

inline bool Sites::is_site_first(Site &s) const
{ return (s.x==0 && s.y==0 && s.z==0); }

inline bool Sites::is_site_last(Site &s) const
{ return (s.x==hindex_max && s.y==vindex_max && s.z==dindex_max); }

inline bool Sites::is_bottom_site(Site s) const
{ return (s.y==vindex_max); }

inline bool Sites::is_top_site(Site s) const
{ return (s.y==0); }

inline bool Sites::is_boundary_site(Site s) const
{ return (s.x==0 || s.y==0 || s.z==0 
		|| s.x==hindex_max || s.y==vindex_max || s.z==dindex_max); }

inline float Sites::unif_status(Site s, int d) const 
{ return *(unif+(4*(s.z*fsize+s.x*vsize+s.y))+d); }

inline bool Sites::edge_status(Site s, int d) const 
{ return *(edges+(4*(s.z*fsize+s.x*vsize+s.y))+d); }

inline bool Sites::mark_status(Site s) const 
{ return *(marks+((s.z*fsize+s.x*vsize+s.y))); }

inline bool Sites::site_status(Site s) const 
{ return *(sites+((s.z*fsize+s.x*vsize+s.y))); }

inline bool Sites::site_status(int i, int j) const 
{ return *(sites_position+i*vsize+j); }

inline void Sites::push_open_neighbours_of_site(Site *current_site, queue<Site> *neighbours) 
{
	static Site neighbour;

	neighbour.x=current_site->x+1;
	neighbour.y=current_site->y;
	neighbour.z=current_site->z;
	if (neighbour.x<hsize) {
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} 

	neighbour.x=current_site->x-1;
	neighbour.y=current_site->y;
	neighbour.z=current_site->z;
	if (neighbour.x!=-1) {
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} 

	neighbour.x=current_site->x;
	neighbour.y=current_site->y+1;
	neighbour.z=current_site->z;
	if (neighbour.y<vsize) {
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} 

	neighbour.x=current_site->x;
	neighbour.y=current_site->y-1;
	neighbour.z=current_site->z;
	if (neighbour.y!=-1) {
		if (site_status(neighbour))
		neighbours->push(neighbour);
	}

	neighbour.x=current_site->x;
	neighbour.y=current_site->y;
	neighbour.z=current_site->z-1;
	if (neighbour.z!=-1) {
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} 

	neighbour.x=current_site->x;
	neighbour.y=current_site->y;
	neighbour.z=current_site->z+1;
	if (neighbour.z!=depth) {
		if (site_status(neighbour))
		neighbours->push(neighbour);
	}
}

inline void Sites::push_open_neighbours_of_bulk_site(Site *current_site, queue<Site> *neighbours) 
{
	static Site neighbour;

	neighbour.y=current_site->y;
	neighbour.z=current_site->z;
	neighbour.x=current_site->x+1;
	if (site_status(neighbour))
	neighbours->push(neighbour);
	neighbour.x=current_site->x-1;
	if (site_status(neighbour))
	neighbours->push(neighbour);
	neighbour.x=current_site->x;
	neighbour.y=current_site->y+1;
	if (site_status(neighbour))
	neighbours->push(neighbour);
	neighbour.y=current_site->y-1;
	if (site_status(neighbour))
	neighbours->push(neighbour);
	neighbour.y=current_site->y;
	neighbour.z=current_site->z-1;
	if (site_status(neighbour))
	neighbours->push(neighbour);
	neighbour.z=current_site->z+1;
	if (site_status(neighbour))
	neighbours->push(neighbour);
}
	
inline void Sites::push_open_neighbours_of_boundary_site(Site *current_site, queue<Site> *neighbours) 
{
	static Site neighbour;

	neighbour.x=current_site->x+1;
	neighbour.y=current_site->y;
	neighbour.z=current_site->z;
	if (neighbour.x<hsize) {
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} else {
		switch (bc) {
			case free:
			break;
			case wired:
	if (depth>2) {
	for (int i=0; i<hsize; i++) {
		for (int j=0; j<vsize; j++) {
		neighbour.x=i;
		neighbour.y=j;
		neighbour.z=0;
		if (site_status(neighbour))
		neighbours->push(neighbour);
		neighbour.z=dindex_max;
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} } }
	for (int i=0; i<hsize; i++) {
		for (int k=0; k<depth; k++) {
		neighbour.x=i;
		neighbour.z=k;
		neighbour.y=0;
		if (site_status(neighbour))
		neighbours->push(neighbour);
		neighbour.y=vindex_max;
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} }
	for (int j=0; j<vsize; j++) {
		for (int k=0; k<depth; k++) {
		neighbour.y=j;
		neighbour.z=k;
		neighbour.x=0;
		if (site_status(neighbour))
		neighbours->push(neighbour);
		neighbour.x=hindex_max;
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} }
			break;
			case toric:
				neighbour.x=0;
				if (site_status(neighbour))
				neighbours->push(neighbour);
			break;
			case Dobrushin:
			break;
	} }

	neighbour.x=current_site->x-1;
	neighbour.y=current_site->y;
	neighbour.z=current_site->z;
	if (neighbour.x!=-1) {
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} else {
		switch (bc) {
			case free:
			break;
			case wired:
	if (depth>2) {
	for (int i=0; i<hsize; i++) {
		for (int j=0; j<vsize; j++) {
		neighbour.x=i;
		neighbour.y=j;
		neighbour.z=0;
		if (site_status(neighbour))
		neighbours->push(neighbour);
		neighbour.z=dindex_max;
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} } }
	for (int i=0; i<hsize; i++) {
		for (int k=0; k<depth; k++) {
		neighbour.x=i;
		neighbour.z=k;
		neighbour.y=0;
		if (site_status(neighbour))
		neighbours->push(neighbour);
		neighbour.y=vindex_max;
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} }
	for (int j=0; j<vsize; j++) {
		for (int k=0; k<depth; k++) {
		neighbour.y=j;
		neighbour.z=k;
		neighbour.x=0;
		if (site_status(neighbour))
		neighbours->push(neighbour);
		neighbour.x=hindex_max;
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} }
			break;
			case toric:
				neighbour.x=hindex_max;
				if (site_status(neighbour))
				neighbours->push(neighbour);
			break;
			case Dobrushin:
			break;
	} }

	neighbour.x=current_site->x;
	neighbour.y=current_site->y+1;
	neighbour.z=current_site->z;
	if (neighbour.y<vsize) {
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} else {
		switch (bc) {
			case free:
			break;
			case wired:
	if (depth>2) {
	for (int i=0; i<hsize; i++) {
		for (int j=0; j<vsize; j++) {
		neighbour.x=i;
		neighbour.y=j;
		neighbour.z=0;
		if (site_status(neighbour))
		neighbours->push(neighbour);
		neighbour.z=dindex_max;
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} } }
	for (int i=0; i<hsize; i++) {
		for (int k=0; k<depth; k++) {
		neighbour.x=i;
		neighbour.z=k;
		neighbour.y=0;
		if (site_status(neighbour))
		neighbours->push(neighbour);
		neighbour.y=vindex_max;
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} }
	for (int j=0; j<vsize; j++) {
		for (int k=0; k<depth; k++) {
		neighbour.y=j;
		neighbour.z=k;
		neighbour.x=0;
		if (site_status(neighbour))
		neighbours->push(neighbour);
		neighbour.x=hindex_max;
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} }
			break;
			case toric:
				neighbour.y=0;
				if (site_status(neighbour))
				neighbours->push(neighbour);
			break;
			case Dobrushin:
	for (int i=0; i<hsize; i++) {
		for (int k=0; k<depth; k++) {
		neighbour.x=i;
		neighbour.z=k;
		neighbour.y=vindex_max;
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} }
			break;
	} }

	neighbour.x=current_site->x;
	neighbour.y=current_site->y-1;
	neighbour.z=current_site->z;
	if (neighbour.y!=-1) {
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} else {
		switch (bc) {
			case free:
			break;
			case wired:
	if (depth>2) {
	for (int i=0; i<hsize; i++) {
		for (int j=0; j<vsize; j++) {
		neighbour.x=i;
		neighbour.y=j;
		neighbour.z=0;
		if (site_status(neighbour))
		neighbours->push(neighbour);
		neighbour.z=dindex_max;
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} } }
	for (int i=0; i<hsize; i++) {
		for (int k=0; k<depth; k++) {
		neighbour.x=i;
		neighbour.z=k;
		neighbour.y=0;
		if (site_status(neighbour))
		neighbours->push(neighbour);
		neighbour.y=vindex_max;
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} }
	for (int j=0; j<vsize; j++) {
		for (int k=0; k<depth; k++) {
		neighbour.y=j;
		neighbour.z=k;
		neighbour.x=0;
		if (site_status(neighbour))
		neighbours->push(neighbour);
		neighbour.x=hindex_max;
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} }
			break;
			case toric:
				neighbour.y=vindex_max;
				if (site_status(neighbour))
				neighbours->push(neighbour);
			break;
			case Dobrushin:
	for (int i=0; i<hsize; i++) {
		for (int k=0; k<depth; k++) {
		neighbour.x=i;
		neighbour.z=k;
		neighbour.y=0;
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} }
			break;
	} }

	neighbour.x=current_site->x;
	neighbour.y=current_site->y;
	neighbour.z=current_site->z-1;
	if (neighbour.z!=-1) {
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} else {
		switch (bc) {
			case free:
			break;
			case wired:
	if (depth>2) {
	for (int i=0; i<hsize; i++) {
		for (int j=0; j<vsize; j++) {
		neighbour.x=i;
		neighbour.y=j;
		neighbour.z=0;
		if (site_status(neighbour))
		neighbours->push(neighbour);
		neighbour.z=dindex_max;
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} } }
	for (int i=0; i<hsize; i++) {
		for (int k=0; k<depth; k++) {
		neighbour.x=i;
		neighbour.z=k;
		neighbour.y=0;
		if (site_status(neighbour))
		neighbours->push(neighbour);
		neighbour.y=vindex_max;
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} }
	for (int j=0; j<vsize; j++) {
		for (int k=0; k<depth; k++) {
		neighbour.y=j;
		neighbour.z=k;
		neighbour.x=0;
		if (site_status(neighbour))
		neighbours->push(neighbour);
		neighbour.x=hindex_max;
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} }
			break;
			case toric:
				neighbour.z=dindex_max;
				if (site_status(neighbour))
				neighbours->push(neighbour);
			break;
			case Dobrushin:
			break;
	} }

	neighbour.x=current_site->x;
	neighbour.y=current_site->y;
	neighbour.z=current_site->z+1;
	if (neighbour.z!=depth) {
		if (site_status(neighbour))
		neighbours->push(neighbour);
	}else {
		switch (bc) {
			case free:
			break;
			case wired:
	if (depth>2) {
	for (int i=0; i<hsize; i++) {
		for (int j=0; j<vsize; j++) {
		neighbour.x=i;
		neighbour.y=j;
		neighbour.z=0;
		if (site_status(neighbour))
		neighbours->push(neighbour);
		neighbour.z=dindex_max;
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} } }
	for (int i=0; i<hsize; i++) {
		for (int k=0; k<depth; k++) {
		neighbour.x=i;
		neighbour.z=k;
		neighbour.y=0;
		if (site_status(neighbour))
		neighbours->push(neighbour);
		neighbour.y=vindex_max;
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} }
	for (int j=0; j<vsize; j++) {
		for (int k=0; k<depth; k++) {
		neighbour.y=j;
		neighbour.z=k;
		neighbour.x=0;
		if (site_status(neighbour))
		neighbours->push(neighbour);
		neighbour.x=hindex_max;
		if (site_status(neighbour))
		neighbours->push(neighbour);
	} }
			break;
			case toric:
				neighbour.z=0;
				if (site_status(neighbour))
				neighbours->push(neighbour);
			break;
			case Dobrushin:
			break;
	} }
}

inline int Sites::random_hindex() 
{ return mtrand_hindex.randInt(hindex_max); }

inline int Sites::random_vindex() 
{ return mtrand_vindex.randInt(vindex_max); }

inline int Sites::random_dindex() 
{ return mtrand_dindex.randInt(dindex_max); }

inline int Sites::random_hindex0() 
{ return mtrand_hindex0.randInt(hindex_max); }

inline int Sites::random_vindex0() 
{ return mtrand_vindex0.randInt(vindex_max); }

inline int Sites::random_dindex0() 
{ return mtrand_dindex0.randInt(dindex_max); }

inline int Sites::random_hindex1() 
{ return mtrand_hindex1.randInt(hindex_max); }

inline int Sites::random_vindex1() 
{ return mtrand_vindex1.randInt(vindex_max); }

inline int Sites::random_dindex1() 
{ return mtrand_dindex1.randInt(dindex_max); }

inline void Sites::set_mark(Site s, bool v)
{ *(marks+((s.z*fsize+s.x*vsize+s.y)))=v; }

inline void Sites::set_site(Site s, bool v)
{ *(sites+((s.z*fsize+s.x*vsize+s.y)))=v; }

inline void Sites::set_unif(Site s, int d, float v)
{ *(unif+(4*(s.z*fsize+s.x*vsize+s.y))+d)=v; }

inline void Sites::set_edge(Site s, int d, bool v)
{ *(edges+(4*(s.z*fsize+s.x*vsize+s.y))+d)=v; }

void Sites::init(int h, int v, int d, int pos=0) {
	hsize=h;
	vsize=v;
	fsize=h*v;
	depth=d;
	position=pos;
	hindex_max=h-1;
	vindex_max=v-1;
	dindex_max=d-1;
	delete [] sites;	
	delete [] marks;	
	delete [] unif;	
	delete [] edges;	
	sites=new bool[h*v*d];
	sites_position=sites+position*fsize; 
	marks=new bool[h*v*d];
	unif=new float[4*h*v*d];
	edges=new bool[4*h*v*d];

	mtrand_hindex.seed( time(NULL));
	mtrand_vindex.seed( time(NULL)+1);
	mtrand_dindex.seed( time(NULL)+2);
	mtrand_flip.seed( time(NULL)+3);

	mtrand_hindex0.seed( time(NULL)+4);
	mtrand_vindex0.seed( time(NULL)+5);
	mtrand_dindex0.seed( time(NULL)+6);
	mtrand_flip0.seed( time(NULL)+7);

	mtrand_hindex1.seed( time(NULL)+7);
	mtrand_vindex1.seed( time(NULL)+8);
	mtrand_dindex1.seed( time(NULL)+9);
	mtrand_flip1.seed( time(NULL)+10);

	float *u=unif;

	for (int i=0; i<hsize; i++)
		for (int j=0; j<vsize; j++)
			for (int k=0; k<depth; k++) {
				for (int d=0; d<4; d++) {
				*u= mtrand_flip.rand(); 
				u++;
				}
	}

	Site current_site, neighbor;

	for (set_site_to_first(&current_site);
		!is_site_last(current_site);
		increase_site(&current_site)) {
			for (int d=0; d<4; d++) {
			neighbor=neighbor_of_site(&current_site,d);
		set_unif(neighbor, (d+2) % 4, unif_status(current_site,d));	
			}
	}

	bool *s=sites;
	bool *m=marks;
	u=unif;
	bool *e=edges;

	for (int i=0; i<hsize; i++)
		for (int j=0; j<vsize; j++)
			for (int k=0; k<depth; k++) {
				for (int d=0; d<4; d++) {
				*e=(*u<prob); 
				u++; e++;
				}
			*s=true;
			*m=false;
			s++; m++;
			}
}



void Sites::resample() {
	float *u=unif;

	for (int i=0; i<hsize; i++)
		for (int j=0; j<vsize; j++)
			for (int k=0; k<depth; k++) {
				for (int d=0; d<4; d++) {
				*u= mtrand_flip.rand(); 
				u++;
				}
	}

	Site current_site, neighbor;

	for (set_site_to_first(&current_site);
		!is_site_last(current_site);
		increase_site(&current_site)) {
			for (int d=0; d<4; d++) {
			neighbor=neighbor_of_site(&current_site,d);
		set_unif(neighbor, (d+2) % 4, unif_status(current_site,d));	
			}
	}

	bool *s=sites;
	bool *m=marks;
	u=unif;
	bool *e=edges;

	for (int i=0; i<hsize; i++)
		for (int j=0; j<vsize; j++)
			for (int k=0; k<depth; k++) {
				for (int d=0; d<4; d++) {
				*e=(*u<prob); 
				u++; e++;
				}
			*s=true;
			*m=false;
			s++; m++;
			}
}

Labels::Labels() {
	labels=new(int);
}

Labels::~Labels() {
	delete [] labels;	
}

void Labels::set_pos(int pos) {
	position=pos;
	labels_position=labels+position*fsize; 
}

void Labels::init(int h, int v, int d, int pos=0) {
	int *l;

	hsize=h;
	vsize=v;
	fsize=h*v;
	depth=d;
	position=pos;
	hindex_max=h-1;
	vindex_max=v-1;
	dindex_max=d-1;
	label_max=h*v;
	delete [] labels;	
	labels=new int[h*v*d];
	
	labels_position=labels+position*fsize; 

	mtrand_hindex.seed( time(NULL)+11);
	mtrand_vindex.seed( time(NULL)+12);
	mtrand_label.seed( time(NULL)+13);

	Site current_site;

	for (set_site_to_first(&current_site);
		!is_site_last(current_site);
		increase_site(&current_site)) {
			set_label(current_site,0); 
	}
}

void Labels::restart() {
	Site current_site;

	for (set_site_to_first(&current_site);
		!is_site_last(current_site);
		increase_site(&current_site)) {
			set_label(current_site,0); 
	}
}

inline int Labels::label_status(Site s) const 
{ return *(labels+((s.z*fsize+s.x*vsize+s.y))); }

inline int Labels::label_status(int i, int j) const 
{ return *(labels_position+i*vsize+j); }

inline void Labels::set_random_label(Site s) 
{ *(labels+((s.z*fsize+s.x*vsize+s.y)))= 1+mtrand_label.randInt(label_max-1); }
	
inline int Labels::random_label() 
{ return(1+mtrand_label.randInt(label_max-1)); }
	
inline void Labels::set_label(Site s, int v) 
{ *(labels+((s.z*fsize+s.x*vsize+s.y)))= v; }
	
LatticeAlgo::~LatticeAlgo()
{
}

LatticeAlgo::LatticeAlgo()
{ }

inline bool LatticeAlgo::is_changed_sites_locked() {
	return (changed_sites_lock);
}

inline void LatticeAlgo::wait_and_lock_changed_sites() {
	while (waiting);
	while (changed_sites_lock) waiting=true;
	changed_sites_lock=true;
	waiting=false;
}

inline void LatticeAlgo::unlock_changed_sites() {
	changed_sites_lock=false;
}

inline void LatticeAlgo::lock_changed_sites() {
	changed_sites_lock=true;
}

int LatticeAlgo::get_iteration() 
{ return iteration+iteration0+iteration1; }

void LatticeAlgo::init(int h, int v, int d, 
float p, float T, int pos=0, int t=0, int sp=0, bool sync=false, bool w=false,
		int d_r=0, int se=4375U)
{
	iteration=0;
	hsize=h; 
	vsize=v; 
	depth=d;
	fsize=h*v;
	set_prob(p);
	set_temp(T);
	set_pos(pos);
	set_threshold(t);
	speed=sp;
	synchronize=sync;
	wait=w;
	delay_report=d_r;
	seed=se;
	mtrand_algo.seed( time(NULL)+14);

	Sites::init(hsize,vsize,depth,pos);
	Labels::init(hsize,vsize,depth,pos);
	
	int n=0;
	empty_changed_sites();
}

void LatticeAlgo::set_prob(float p) {
	prob=p;
}

void LatticeAlgo::set_threshold(int t) {
	threshold=t;
}

void LatticeAlgo::set_pos(int pos) {
	Sites::set_pos(pos);
	Labels::set_pos(pos);
}

void LatticeAlgo::set_synchronize(bool sync) {
	synchronize=sync;
	if (sync) {
		empty_changed_sites();
	}
}

void LatticeAlgo::set_wait(bool w) {
	wait=w;
}

void LatticeAlgo::set_speed(int sp) {
	speed=sp;
}

void LatticeAlgo::set_temp(float T) {
	temp=T;
}

void LatticeAlgo::set_field(float hm) {
	field=hm;
}

int LatticeAlgo::run(algo_t al, int id, int max_iteration=-1) {
	switch(al) {
		case Static: if (id==0) static0(id, max_iteration);
			else if (id==1) static0(id, max_iteration);
			break;
		case Dynamic: if (id==0) growth0(id, max_iteration);
			else if (id==1) growth0(id, max_iteration);
			break;
	}
	return 0;
}

/* the access to changed_sites is supposed to be locked
by the calling procedure */
bool LatticeAlgo::next_site(int *i, int *j) {
	wait_and_lock_changed_sites();
	if (!changed_sites_lock) cout << "aie!" << endl;
	changed_site.z=-1;
	while (changed_site.z!=position && !changed_sites.empty()) {
		changed_site=changed_sites.front();
		changed_sites.pop();
	}
	unlock_changed_sites();
	if (changed_site.z==position) {
		*i=changed_site.x;
		*j=changed_site.y;
		return true;
	} else { return false; }
}

void LatticeAlgo::wait_or_synchronize() {
	static int iter=0;
	if (iter++>speed) {
		if (synchronize) {
		while(!is_changed_sites_empty()) { usleep(0); } 
		}
		if (wait) { usleep(100-speed); }
		iter=0;
	}
}

bool LatticeAlgo::is_changed_sites_empty() {
	wait_and_lock_changed_sites();
	if (changed_sites.empty()) {
		unlock_changed_sites();
		return(true);
	} else {
		unlock_changed_sites();
		return(false);
	}
		
}

void LatticeAlgo::empty_changed_sites() {
	lock_changed_sites();
	while(!changed_sites.empty()) {
		changed_sites.pop(); }
	unlock_changed_sites();
	waiting=false;
}

void LatticeAlgo::set_label_and_push_site(Site s, int v) {
	set_label(s,v);
	if (synchronize) { 
		wait_and_lock_changed_sites();
	if (!changed_sites_lock) cout << "aie!" << endl;
		changed_sites.push(s); 
		unlock_changed_sites();
	}

}

int LatticeAlgo::mark_cluster(Site s, queue<Site> *frontier, int l) {
	int empty_label;
	int current_label;
	int cluster_size;
	Site current_site, neighbor;

		cluster_size=0;
			current_site=s;
			frontier->push(current_site);

			while (! frontier->empty()) { 
        			current_site=frontier->front();
        			frontier->pop(); 
	if (!mark_status(current_site)) {
		cluster_size++;
		set_mark(current_site, true);
		set_label_and_push_site(current_site, l);
		for (int d=0; d<4; d++) {
			if (edge_status(current_site,d)) {
			neighbor=neighbor_of_site(&current_site,d);
			if (!mark_status(neighbor)) {
			frontier->push(neighbor);
			} }
		} } }
	return(cluster_size);
}

int LatticeAlgo::labelize_and_grow_cluster(Site s, int l,
		queue<Site> *frontier) {
	int empty_label;
	int current_label;
	int cluster_size;
	Site current_site, neighbor;

		cluster_size=0;
		current_label=l;
			current_site=s;
			frontier->push(current_site);

/* -1 means the distance from the starting point is increased by 1 */
		current_site.x=-1;
		current_site.y=-1;
		current_site.z=-1;
		frontier->push(current_site);

			while (! frontier->empty()) { 
        			current_site=frontier->front();
        			frontier->pop(); 
			if (current_site.x<0) {
/* distance from starting point has increased by one
   so if asked we wait for the screen to proceed */
				wait_or_synchronize();
				if  (!frontier->empty()) { 
				frontier->push(current_site); }
			} else {
/* process the current point */	
	if (label_status(current_site)!=current_label) {
		cluster_size++;
		set_label_and_push_site(current_site, current_label);
		for (int d=0; d<4; d++) {
			if (edge_status(current_site,d)) {
			neighbor=neighbor_of_site(&current_site,d);
			if (label_status(neighbor)!=current_label) {
			frontier->push(neighbor);
			} }
		} } } }
	return(cluster_size);
}

int LatticeAlgo::labelize_cluster(Site s, int l,
		queue<Site> *frontier) {
	int empty_label;
	int current_label;
	int cluster_size;
	Site current_site, neighbor;

		cluster_size=0;
		current_label=l;
			current_site=s;
			frontier->push(current_site);

			while (! frontier->empty()) { 
        			current_site=frontier->front();
        			frontier->pop(); 
	if (label_status(current_site)!=current_label) {
		cluster_size++;
		set_label_and_push_site(current_site, current_label);
		for (int d=0; d<4; d++) {
			if (edge_status(current_site,d)) {
			neighbor=neighbor_of_site(&current_site,d);
			if (label_status(neighbor)!=current_label) {
			frontier->push(neighbor);
			} }
		} } }
	return(cluster_size);
}

void LatticeAlgo::clear_marks() {
	Site current_site;

	for (set_site_to_first(&current_site);
		!is_site_last(current_site);
		increase_site(&current_site)) {
			set_mark(current_site,false);
		}
}

float LatticeAlgo::update_site_and_labels_with_prob() {
	static float current_prob=prob;
	Site current_site;

	if (current_prob!=prob) {
	for (set_site_to_first(&current_site);
		!is_site_last(current_site);
		increase_site(&current_site)) {
	set_mark(current_site,false);
	for (int d=0; d<4; d++) {
		if (unif_status(current_site,d)<prob) {
			set_edge(current_site, d, true);
		} else {
			set_edge(current_site, d, false);
		}
	} } }

	for (set_site_to_first(&current_site);
		!is_site_last(current_site);
		increase_site(&current_site)) {
			for (int d=0; d<4; d++) {
			Site neighbor=neighbor_of_site(&current_site,d);
	/*if(edge_status(neighbor, (d+2) % 4)!=edge_status(current_site,d))	
		{ cout << current_site.x << " " << current_site.y << " " <<
		neighbor.x << " " << neighbor.y << " " <<
		unif_status(neighbor, (d+2) % 4) << " " <<
unif_status(current_site,d)	<< endl; }*/
			}
	}
	float delta=current_prob-prob;
	current_prob=prob;
	return(delta);
}

int LatticeAlgo::update_site_and_labels_with_threshold() {
	static int current_threshold=threshold;
	Site current_site;

	if (current_threshold!=threshold) {
	for (set_site_to_first(&current_site);
		!is_site_last(current_site);
		increase_site(&current_site)) {
		set_mark(current_site,false);
	} }

	int delta=current_threshold-threshold;
	current_threshold=threshold;
	return(delta);
}

int LatticeAlgo::static0(int id, int max_iteration=-1) {
	int cluster_size=0;
	Site current_site;
	queue<Site> frontier;

	clear_marks();

	bool merge=false;
	int number_of_changes=0;
	iteration0=0;
	while (iteration0!=max_iteration) {
		iteration0++;

	number_of_changes=0;
	for (int i=0; i<hsize*vsize*depth; i++) {
	random_site(&current_site);
		cluster_size=
	mark_cluster(current_site, &frontier,
	(merge && label_status(current_site)>0 ?
		label_status(current_site) : random_label()));
	i+=cluster_size;
	number_of_changes+=cluster_size;
if (cluster_size>0 && cluster_size<threshold) {
	labelize_cluster(current_site, 0, &frontier);
	} 
	}
	wait_or_synchronize();
	
	bool no_change=true;

	if (update_site_and_labels_with_threshold()!=0.) {
		no_change=false; }
	float delta=update_site_and_labels_with_prob();
	merge=(delta<=0.);
	if (delta!=0.) { no_change=false; }

if (no_change && speed>0 && iteration0 % ((100-speed)/10+1)==0) {
		Sites::resample();
		Labels::restart();
	}

	}

	return(iteration0);
}


int LatticeAlgo::growth0(int id, int max_iteration=-1) {
	int cluster_size=0;
	Site current_site;
	queue<Site> frontier;

	clear_marks();

	bool merge=false;
	iteration0=0;
	while (iteration0!=max_iteration) {
		iteration0++;

	for (int i=0; i<hsize*vsize*depth; i++) {
	random_site(&current_site);
	if (site_status(current_site)) {
cluster_size=labelize_and_grow_cluster(current_site, 0, &frontier);
	i+=cluster_size;
	if (cluster_size>=threshold) {
labelize_and_grow_cluster(current_site, random_label(), &frontier);
	} } }
	
	update_site_and_labels_with_threshold();
	update_site_and_labels_with_prob();

	}

	return(iteration0);
}


}


#define NTHREADS 10
pthread_t thread[NTHREADS];
typedef struct {
	int id;
	algo_t al;
} thread_info_t;
thread_info_t thread_info[NTHREADS];
int nthreads=0;

/* the algorithm */
class computation::LatticeAlgo algo;

#define errexit(code,str)                          \
  fprintf(stderr,"(%d) %s: %s\n",code,(str),strerror(code)); \
  exit(1);

/************************************************************************* 
	procedures to launch the threads runnning the simulation code 
**************************************************************************/

void *launch_computation(void *arg) {
  int id=((thread_info_t *) arg)->id;
  algo_t al=((thread_info_t *) arg)->al;
    pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
    pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
	algo.run(al,id);
   return ((void *)NULL);
}

int launch_computation_thread(algo_t al, int num) {
	for (int i=0; i<num; i++) {	
	if (nthreads<NTHREADS) {
    		thread_info[nthreads].id=nthreads;
    		thread_info[nthreads].al=al;
    		if (int errcode=pthread_create(&thread[nthreads], 
			NULL, launch_computation, &thread_info[nthreads])) { 
   		errexit(errcode,"pthread_create");
   		}
    		nthreads++;
	} }
	return nthreads;
}

int stop_computation_threads() {
	    pthread_cancel(thread[0]);
	for (int i=0; i<nthreads; i++) {
	    if (int errcode=pthread_cancel(thread[i])) { 
      		errexit(errcode,"pthread_cancel");
	} }
	nthreads=0;
	return nthreads;
}

namespace interface {

/* the global parameters handled by control_window and data_display */
int algo_speed=0;
bool synchronize=false;
bool wait=false;
int refresh_hsize=10;
int refresh_vsize=10;
int refresh_delay=0;
int threshold_scroll_delay=0;
int depth_scroll_delay=0;
int refresh_number=0;
float prob=0.49;
float temp=2.2691854;
float field=0.0;
gint h_digits=1;
float epsi=0.0;
gint e_digits=1;
int hlength,vlength,depth;
int ncolors;
gfloat refresh_size_value=100;
gfloat depth_ratio_value=1;
/* these are the values given by the widgets, they are percentages */
gfloat threshold_value=0.;
gfloat depth_pos=50.;
/* these are the values given to the algorithm */
int threshold;
int position;
time_t time_start=time(NULL);

string black_color_name="black";
string white_color_name="white";

#define NCOLORS 10 
string color_names[NCOLORS]={"black", "white", "red", "blue", "green",
"orange", "purple", "yellow", "grey", "brown"};

string bdc_name="free";
#define NBDC 4 
string bdc_names[NBDC]={"free", "wired", "toric", "Dobrushin"};

/***********************************************************************
   There are four main objects:
	DataDisplay: in charge of extracting and visualizing the data
	LatticeWindow: the window where the lattice is visualized
	LatticeDrawingArea: the visualized lattice
	ControlWindow: the user interface
************************************************************************/

class DataDisplay;

class LatticeDrawingArea  : public Gtk::DrawingArea
{
  /* Backing pixmap for drawing area */
  Gdk_Pixmap pixmap;
  Gdk_Image image;
  Gdk_GC gc;
  Gdk_GC current_gc;
  Gdk_GC black_gc;
  Gdk_GC white_gc;
  Gdk_Color *colors;
  guint32 *pixels;
  guint32 black_pixel, white_pixel;
  Gdk_Window win;
  Gdk_Visual visual;
  bool connected;
  DataDisplay *data_display;

  virtual gint configure_event_impl (GdkEventConfigure *event);
  virtual gint expose_event_impl (GdkEventExpose *event);
  virtual gint button_press_event_impl (GdkEventButton *event);
  virtual gint motion_notify_event_impl (GdkEventMotion *event);
  void draw_black_point (gint x, gint y);
  void draw_white_point (gint x, gint y);
  void draw_point (int x, int y, int col);
  void put_black_pixel (gint x, gint y);
  void put_white_pixel (gint x, gint y);
  void put_pixel (gint x, gint y, int col);
  void draw_image(gint x, gint y, gint w, gint h); 
  void set_connected_false();
  void set_connected_true();
public:
  LatticeDrawingArea (DataDisplay *d_d);
  ~LatticeDrawingArea ();
  void erase();
   void set_current_color(int no);
   void set_black_color();
   void set_white_color();
   friend class ControlWindow;
   friend class DataDisplay;
};


class LatticeWindow : public Gtk::Window
{
    Gtk::VBox vbox;
    DataDisplay *data_display;
  public:  
    LatticeDrawingArea drawing_area;
    LatticeWindow (DataDisplay *d_d);
    ~LatticeWindow ();
  }; 

class DataDisplay
{
    int connect_refresh_points_synchronized;
    int connect_refresh_points_with_delay;
    int connect_refresh_points_without_delay;
    bool connected;
    MTRand mtrand;
    LatticeWindow lattice_window;
    bool control_window_ready;
public:
    ControlWindow *control_window;
    DataDisplay();
    ~DataDisplay();
    void init();
    void run();
    void restart();
    void stop();
    void erase();
    void show();
    void connect_synchronized();
    void connect_with_delay();
    void connect_without_delay();
    void set_black_color();
    void set_white_color();
    void set_connected_false();
    void set_connected_true();
    void start_connection();
    void end_connection();
    void refresh_size_update();
    void threshold_update();
    void position_update();
    void depth_update();
    int DataDisplay::refresh_points_synchronized();
    int DataDisplay::refresh_points_without_delay(); 
    int DataDisplay::refresh_points_with_delay(); 
   friend class ControlWindow;
};

class ControlWindow : public Gtk::Window
  {
    Gtk::SpinButton *spinner_T_val;
    Gtk::SpinButton *spinner_T_pre;
    Gtk::HScale *T_digits;
    Gtk::SpinButton *spinner_he_val;
    Gtk::SpinButton *spinner_he_pre;
    Gtk::HScale *he_digits;
    Gtk::SpinButton *spinner_p_val;
    Gtk::SpinButton *spinner_p_pre;
    Gtk::HScale *p_digits;
    Gtk::RadioButton rb_server, rb_client;
    Gtk::RadioButton rb_noise, rb_algo;
    Gtk::RadioButton rb_random, rb_sweep;
    Gtk::RadioButton rb_constant, rb_rndh;
    Gtk::RadioButton rb_free, rb_wait, rb_synchro;
    Gtk::CheckButton ck_field;
    Gtk::CheckButton ck_threshold_scroll;
    Gtk::CheckButton ck_depth_scroll;
    Gtk::CheckButton ck_auto;
    Gtk::CheckButton ck_bipro;
    Gtk::RadioButton rb_static, rb_threshold, rb_dynamic;
    Gtk::RadioButton rb_sites, rb_labels;
    Gtk::Adjustment adj_ratio;
    Gtk::Adjustment adj_threshold;
    Gtk::Adjustment adj_pos;
    Gtk::Adjustment adj_threshold_scroll_delay;
    Gtk::Adjustment adj_depth_scroll_delay;
    Gtk::Adjustment adj_speed;
    Gtk::Adjustment adj_size;
    Gtk::Adjustment adj_delay;
    Gtk::Button initer;
    Gtk::Button runner;
    Gtk::Button restarter;
    Gtk::Button stopper;
    Gtk::Button eraser;
    Gtk::Button threshold_increaser;
    Gtk::Button threshold_decreaser;
    Gtk::Button depth_increaser;
    Gtk::Button depth_decreaser;
    Gtk::Button button;
    Gtk::HScale *threshold_wt;
    Gtk::HScale *threshold_scroll_delay_wt;
    Gtk::HScale *depth_ratio_wt;
    Gtk::HScale *depth_pos_wt;
    Gtk::HScale *depth_scroll_delay_wt;
    Gtk::HScale *algo_speed_wt;
    Gtk::HScale *refresh_size_wt;
    Gtk::HScale *refresh_delay_wt;
    Gtk::Label *prob_label;
    Gtk::Frame *frame_stats;
    Gtk::Frame *frame_depth;
    Gtk::Label *threshold_label;
    Gtk::Label *visu_label;
    Gtk::Label *runtime_label;
    Gtk::Label *iter_label;
    Gtk::Label *refresh_label;
    Gtk::Label *minus_color_label;
    Gtk::Label *plus_color_label;
    Gtk::Label *bdc_label;
    Gtk::Label *para_label;
    Gtk::VBox vbox;
    Gtk::Connection connection;
    DataDisplay *data_display;
	
  public:  
    ControlWindow (DataDisplay *d_d);
    ~ControlWindow ();
    void set_button_states(bool i, bool r, bool t, bool s);
    void set_mode_states(bool f, bool w, bool s);
    void update_prob();
    void update_temp();
    void update_field();
    void update_depth();
    void update_threshold();
    void update_pos();
    void p_digits_cb(Gtk::Adjustment *adj);
    void T_digits_cb(Gtk::Adjustment *adj);
    void he_digits_cb(Gtk::Adjustment *adj);
    void p_value_cb( Gtk::SpinButton *spin );
    void T_value_cb( Gtk::SpinButton *spin );
    void algo_speed_cb(Gtk::Adjustment *adj);
    void he_value_cb( Gtk::SpinButton *spin );
    void threshold_cb(Gtk::Adjustment *adj);
    void increase_threshold();
    void decrease_threshold();
    void depth_ratio_cb(Gtk::Adjustment *adj);
    void depth_pos_cb(Gtk::Adjustment *adj);
    void increase_pos();
    void decrease_pos();
    void refresh_size_cb(Gtk::Adjustment *adj);
    void refresh_delay_cb(Gtk::Adjustment *adj);
    void threshold_scroll_delay_cb(Gtk::Adjustment *adj);
    void depth_scroll_delay_cb(Gtk::Adjustment *adj);
    int statistics();
    void void_minus_color_int_cb(int num);
    void void_plus_color_int_cb(int num);
    void void_bdc_int_cb(int num);
    void field_cb();
    void constant_cb();
    void rndh_cb();
    void algo_cb();
    void synchro_cb();
    void wait_cb();
    void run();
    void restart();
    void init();
    void stop();
    void erase();
    int refresh_points_synchronized();
    int refresh_points_without_delay();
    int refresh_points_with_delay(); 
    void start_connection();
    void end_connection();
   friend class DataDisplay;
  }; 


LatticeDrawingArea::LatticeDrawingArea (DataDisplay *d_d)
    : Gtk::DrawingArea(), pixmap (0)
  {
    set_events (GDK_EXPOSURE_MASK
		| GDK_LEAVE_NOTIFY_MASK
		| GDK_BUTTON_PRESS_MASK
		| GDK_POINTER_MOTION_MASK
		| GDK_POINTER_MOTION_HINT_MASK);
    data_display=d_d;
    connected=false;
    colors = new Gdk_Color[1];
    pixels = new guint32[1];
  }

/***********************************************************************************
   LatticeDrawingArea
************************************************************************************/

LatticeDrawingArea::~LatticeDrawingArea()
{
	delete [] colors;
	delete [] pixels;
}

void LatticeDrawingArea::set_connected_false() {
	connected=false;
}

void LatticeDrawingArea::set_connected_true() {
	connected=true;
}

/* Create a new backing pixmap of the appropriate size */
int LatticeDrawingArea::configure_event_impl (GdkEventConfigure *event)
  {
    data_display->end_connection();
    data_display->stop();

    win = get_window();
    visual = win.get_visual();

    if (pixmap)
      pixmap.release();

    gc = get_style()->get_white_gc();
    pixmap.create(get_window(),  width(), height());

    pixmap.draw_rectangle (gc, true, 0, 0, width(), height());

    hlength=width();
    vlength=height();
    depth=(int)(hlength*depth_ratio_value/100);

    black_gc = Gdk_GC(pixmap);
    Gdk_Color black(black_color_name);
    get_colormap().alloc(black);
    black_gc.set_foreground(black);
    black_pixel=black.get_pixel();

    white_gc = Gdk_GC(pixmap);
    Gdk_Color white(white_color_name);
    get_colormap().alloc(white);
    white_gc.set_foreground(white);
    white_pixel=white.get_pixel();

    current_gc = Gdk_GC(pixmap);

    ncolors=get_colormap().get_system_size();

    delete [] colors; 	 
    delete [] pixels; 	 

    colors = new Gdk_Color[ncolors];
    pixels = new guint32[ncolors];
    colors[0]=black; pixels[0]=black_pixel;
    colors[1]=white; pixels[1]=white_pixel;
    for (int i=2; i<ncolors; i++) {
    	colors[i].set_random();
    	get_colormap().alloc(colors[i]);
        pixels[i]=colors[i].get_pixel();
    }

    image.create(GDK_IMAGE_NORMAL, visual, hlength, vlength);

    data_display->refresh_size_update();
    data_display->threshold_update();
    data_display->depth_update();
    data_display->position_update();
    data_display->init();
    data_display->start_connection();
    data_display->run();
    return true;
  }

/* handling of colors is not clear at all!! */
void LatticeDrawingArea::set_current_color(int no) {
    current_gc.set_foreground(colors[no]);
}


void LatticeDrawingArea::set_black_color() {
    black_gc = Gdk_GC(pixmap);
    Gdk_Color black(black_color_name);
    get_colormap().alloc(black);
    black_gc.set_foreground(black);
    black_pixel=black.get_pixel();
    colors[0]=black;
    pixels[0]=black_pixel;
}

void LatticeDrawingArea::set_white_color() {
    white_gc = Gdk_GC(pixmap);
    Gdk_Color white(white_color_name);
    get_colormap().alloc(white);
    white_gc.set_foreground(white);
    white_pixel=white.get_pixel();
    colors[1]=white;
    pixels[1]=white_pixel;
}

/* Redraw the screen from the backing pixmap */
int LatticeDrawingArea::expose_event_impl (GdkEventExpose *event)
  {
    gc = get_style()->get_fg_gc(get_state());
    win.draw_pixmap(gc ,
		    pixmap,
		    event->area.x, event->area.y,
		    event->area.x, event->area.y,
		    event->area.width, event->area.height);

    return false;
  }

inline void LatticeDrawingArea::draw_black_point (gint x, gint y)
  {
    pixmap.draw_point(black_gc,x,y);
  }

inline void LatticeDrawingArea::draw_white_point (gint x, gint y)
  {
    pixmap.draw_point(white_gc,x,y);
  }

inline void LatticeDrawingArea::draw_point (int x, int y, int col)
  {
    current_gc.set_foreground(colors[col]);
    pixmap.draw_point(current_gc,x,y);
  }

inline void LatticeDrawingArea::put_black_pixel (gint x, gint y)
  {
    image.put_pixel(x,y,black_pixel);
  }

inline void LatticeDrawingArea::put_white_pixel (gint x, gint y)
  {
    image.put_pixel(x,y,white_pixel);
  }

inline void LatticeDrawingArea::put_pixel (gint x, gint y, int col)
{
   image.put_pixel(x,y,pixels[col]);
}

inline void LatticeDrawingArea::draw_image(gint x, gint y, gint w, gint h) {
	pixmap.draw_image(gc, image, x, y, x, y, w, h);
}

gint LatticeDrawingArea::button_press_event_impl (GdkEventButton *event)
  {
    if (event->button == 1 && pixmap)
      draw_white_point (event->x, event->y);

    return true;
  }

gint LatticeDrawingArea::motion_notify_event_impl (GdkEventMotion *event)
  {
    int x, y;
    GdkModifierType state;
    Gdk_Window window(event->window);
    if (event->is_hint)
      window.get_pointer (x, y, state);
    else {
	x = (int)event->x;
	y = (int)event->y;
	state = (GdkModifierType) event->state;
      }
    
    if (state & GDK_BUTTON1_MASK && pixmap)
      draw_white_point (x, y);
  
    return true;
  }

void LatticeDrawingArea::erase()
  {
    // clear pixmap area to white.
    gc = get_style()->get_white_gc();
    pixmap.draw_rectangle (gc, true, 0, 0, width(), height());

    // request a refresh of whole area.
    draw(0);
  }

/***********************************************************************************
   LatticeWindow
************************************************************************************/

LatticeWindow::LatticeWindow (DataDisplay *d_d)
    :  Gtk::Window(GTK_WINDOW_TOPLEVEL),
	vbox (false, 0), drawing_area(d_d)
  {
    data_display=d_d;

    set_title("Lattice");
    add (vbox);

    vbox.pack_start (drawing_area, true, true, 0);

    destroy.connect(Gtk::Main::quit.slot());

    drawing_area.show();
    vbox.show();
  }

LatticeWindow::~LatticeWindow()
  {}

/***********************************************************************************
   ControlWindow
************************************************************************************/

void ControlWindow::run() {
	data_display->run();
}

void ControlWindow::restart() {
	data_display->restart();
}

void ControlWindow::init() {
	data_display->init();
}

void ControlWindow::stop() {
	data_display->stop();
}

void ControlWindow::erase() {
	data_display->erase();
}

void ControlWindow::T_value_cb( Gtk::SpinButton *spin )
{
	temp=spin->get_value_as_float (); 
	update_temp();
}

void ControlWindow::he_value_cb( Gtk::SpinButton *spin )
{
	if (rb_constant.get_active()) 
		{ field=spin->get_value_as_float(); }
	if (rb_rndh.get_active()) 
		{ epsi=spin->get_value_as_float(); }
	update_field();
}

void ControlWindow::p_value_cb( Gtk::SpinButton *spin )
{
	prob=spin->get_value_as_float (); 
	update_prob();
}

void ControlWindow::void_minus_color_int_cb(int num)
{
	black_color_name=color_names[num];
	minus_color_label->set_text ("is "+black_color_name);

	data_display->set_black_color();
}

void ControlWindow::void_plus_color_int_cb(int num)
{
	white_color_name=color_names[num];
  	plus_color_label->set_text ("is "+white_color_name);
	data_display->set_white_color();
}

void ControlWindow::synchro_cb() {
	if (rb_synchro.get_active()) { 
	  data_display->connect_refresh_points_synchronized=1;
	  data_display->connect_refresh_points_with_delay=0;
	  data_display->connect_refresh_points_without_delay=0;
		synchronize=true;
		algo.set_synchronize(true); }
	else { 
	  data_display->connect_refresh_points_synchronized=0;
	  data_display->connect_refresh_points_with_delay=0;
	  data_display->connect_refresh_points_without_delay=1;
		synchronize=false;
		algo.set_synchronize(false); }
	//end_connection();
	start_connection();
}

void ControlWindow::wait_cb() {
	if (rb_wait.get_active()) { 
		wait=true;
		algo.set_wait(true); }
	else {
		wait=false;
		algo.set_wait(false); }
}

void ControlWindow::algo_cb() {
  data_display->stop();
  data_display->run();
}

void ControlWindow::field_cb() {
	algo_cb();
	if (rb_constant.get_active()) { ; }
	if (rb_rndh.get_active()) { ; }
}

void ControlWindow::constant_cb() {
	para_label->set_text ("h=");
 	spinner_he_val->set_digits(h_digits);
(he_digits->get_adjustment())->set_value((gfloat)h_digits);
(spinner_he_val->get_adjustment())->set_step_increment(pow(10, (int)-h_digits));
(spinner_he_val->get_adjustment())->set_value(field);
	field_cb();
}

void ControlWindow::rndh_cb() {
	para_label->set_text ("e=");
 	spinner_he_val->set_digits(e_digits);
(he_digits->get_adjustment())->set_value((gfloat)e_digits);
(spinner_he_val->get_adjustment())->set_step_increment(pow(10, (int)-e_digits));
(spinner_he_val->get_adjustment())->set_value(epsi);
	field_cb();
}

void ControlWindow::void_bdc_int_cb(int num)
{
	bdc_name=bdc_names[num];
	bdc_label->set_text ("are "+bdc_name);
	algo.set_bc(num);
}

ControlWindow::ControlWindow (DataDisplay *d_d)
    :  Gtk::Window(GTK_WINDOW_TOPLEVEL),
       rb_server("server"),
       rb_client("client"),
       rb_noise("noise"),
       rb_algo("algo   "),
       rb_random("rand"),
       rb_sweep("sweep"),
       rb_constant("constant"),
       rb_rndh("random"),
       rb_free("free"),
       rb_wait("wait"),
       rb_synchro("synchronize"),
       ck_field("toggle field"),
       ck_threshold_scroll("scroll"),
       ck_depth_scroll("scroll"),
       ck_auto("auto"),
       ck_bipro("bipro"),
       rb_static("static"),
       rb_threshold("threshold"),
       rb_dynamic("grow"),
       rb_sites("sites"),
       rb_labels("labels"),
	adj_ratio(depth_ratio_value, 1, 100),
	adj_threshold(50, 0, 100),
	adj_pos(50, 0, 100),
	adj_threshold_scroll_delay(10, 0, 100),
	adj_depth_scroll_delay(10, 0, 100),
	adj_speed(100, 0, 100),
	adj_size(100, 1, 100),
	adj_delay(0, 0, 100),
       initer("init"),
       runner("run"),
       restarter("restart"),
       stopper("stop"),
       eraser("erase"),
       threshold_increaser("+"),
       threshold_decreaser("-"),
       depth_increaser("+"),
       depth_decreaser("-"),
       button ("quit")
  {
  data_display=d_d;
  data_display->show();
  
  data_display->control_window=this;
  data_display->control_window_ready=true;

  set_title("Gnu Percolation");

  Gtk::VBox *main_vbox = manage( new Gtk::VBox(false, 0) );
  add (*main_vbox);

  Gtk::Frame *frame_temp = manage( new Gtk::Frame ("T (Tc=?)") );
  //main_vbox->pack_start (*frame_temp, true, true, 10);

	  Gtk::HBox *box10=manage(new Gtk::HBox(false,10));
	  frame_temp->add (*box10);

	  Gtk::HBox *box101=manage(new Gtk::HBox(false,10));
	  box10->pack_start(*box101,false,false,0);

  	  	box101->pack_start (*manage( new Gtk::Label ("T=", 0, 0.5) )
		, false, true, 0);
  
		Gtk::Adjustment *adj = manage( new Gtk::Adjustment (2.2691854, 0.0, 10.0, 0.1, 0.1, 0.0) );

  		spinner_T_val = manage( new Gtk::SpinButton (*adj, 0, 1) );
  		spinner_T_val->set_wrap (false);
  		spinner_T_val->set_usize (65 , 0);
  		adj->value_changed.connect(bind(slot(this, &ControlWindow::T_value_cb),spinner_T_val));
  		box101->pack_start (*spinner_T_val, false, true, 0);

	  Gtk::HBox *box102=manage(new Gtk::HBox(false,10));
	  box10->pack_start(*box102,false,false,0);

	  	Gtk::HBox *box00=manage(new Gtk::HBox(false,0));
	  	box102->pack_start(*manage(new Gtk::Label("digits: ",0)),false,false);
	
	  	Gtk::Adjustment *adj1=manage(new Gtk::Adjustment(1, 1, 4));
	  	T_digits=manage(new Gtk::HScale(*adj1)); 
 	  	T_digits->set_digits(0);
	  	adj1->value_changed.connect(bind(slot(this,&ControlWindow::T_digits_cb),adj1));
	  	box102->pack_start(*T_digits,true,true);

  Gtk::Frame *frame_algo = manage( new Gtk::Frame ("algorithm") );
  main_vbox->pack_start (*frame_algo, true, true, 10);

	  Gtk::VBox *box_dyna=manage(new Gtk::VBox(false,0));
	  frame_algo-> add (*box_dyna);

	  Gtk::HBox *box_algo=manage(new Gtk::HBox(false,0));
	  box_dyna->pack_start(*box_algo);

		  rb_static.set_group(rb_dynamic.group());
		  //rb_threshold.set_group(rb_dynamic.group());
		  rb_static.set_active(true);
  rb_static.clicked.connect(slot(this,&ControlWindow::algo_cb));
  rb_threshold.clicked.connect(slot(this,&ControlWindow::algo_cb));
  rb_dynamic.clicked.connect(slot(this,&ControlWindow::algo_cb));

		  box_algo->pack_start(rb_static);
		  //box_algo->pack_start(rb_threshold);
		  box_algo->pack_start(rb_dynamic);

  //box_algo->pack_start (*manage(new Gtk::VSeparator()),false, true,0); 
	  //Gtk::HBox *box934=manage(new Gtk::HBox(false,10));
	  //box_algo->pack_start(*box934);

		  rb_sites.set_group(rb_labels.group());
		  rb_sites.set_active(false);

		  //box_algo->pack_start(rb_sites);
		  //box_algo->pack_start(rb_labels);

  box_dyna->pack_start (*manage(new Gtk::HSeparator()),false, true,0); 
	  Gtk::HBox *box99=manage(new Gtk::HBox(false,10));
	  //box_algo->pack_start(*box99);

  Gtk::MenuBar *bdc_menubar = manage( new Gtk::MenuBar());

  {
     using namespace Gtk;
     using namespace Menu_Helpers;

     Menu    *bdc_menu      = manage( new Menu());
     MenuList& bdc_list_file     = bdc_menu->items();

	for (int i=0; i<NBDC; i++) {
     bdc_list_file.push_back(MenuElem(bdc_names[i], bind(slot(this, &ControlWindow::void_bdc_int_cb), i)));
	}

     MenuElem bdc_elem("boundary conditions",*bdc_menu);
     bdc_menubar->items().push_front(bdc_elem);

   }
   box99->pack_start (*bdc_menubar, false, false);

  bdc_label = manage( new Gtk::Label ("") );
  bdc_label->set_text ("are "+bdc_name);
  box99->pack_start (*bdc_label, false, true, 0);

	  Gtk::HBox *box888=manage(new Gtk::HBox(false,10));
	  //box_dyna->pack_start(*box888);
		  rb_free.set_active(true);
		  box888->pack_start(rb_free);

		  rb_wait.set_active(false);
		  wait=false;
  		rb_wait.clicked.connect(slot(this,&ControlWindow::wait_cb));
		  box888->pack_start(rb_wait);
		  rb_wait.set_group(rb_free.group());

		  rb_synchro.set_active(false);
		  synchronize=false;
  		rb_synchro.clicked.connect(slot(this,&ControlWindow::synchro_cb));
		  box888->pack_start(rb_synchro);
		  rb_synchro.set_group(rb_free.group());

//box_dyna->pack_start (*manage(new Gtk::HSeparator()),false, true,0); 
	  Gtk::HBox *box999=manage(new Gtk::HBox(false,10));
	  box_dyna->pack_start(*box999);
  box999->pack_start(*manage(new Gtk::Label("speed: ",0)),false,false);

		  adj_speed.set_value(0);
		  algo_speed=0;
		  algo_speed_wt=manage(new Gtk::HScale(adj_speed)); 
		  algo_speed_wt->set_digits(0);
		  adj_speed.value_changed.connect(bind(slot(this,&ControlWindow::algo_speed_cb),&adj_speed));
		  box999->pack_start(*algo_speed_wt,true,true);


	  Gtk::HBox *box88=manage(new Gtk::HBox(false,10));
	  //box_dyna->pack_start(*box88);

	  ck_field.set_active(false);
	  box88->pack_start(ck_field);

  	  ck_field.clicked.connect(slot(this,&ControlWindow::field_cb));


	  box88->pack_start (*manage(new Gtk::VSeparator()),false, true,0); 
		  rb_constant.set_group(rb_rndh.group());
		  rb_constant.set_active(true);
	rb_constant.clicked.connect(slot(this,&ControlWindow::constant_cb));
	rb_rndh.clicked.connect(slot(this,&ControlWindow::rndh_cb));

		  Gtk::HBox *box825=manage(new Gtk::HBox(false,0));
	  //box88->pack_start (*box825, false, false, 0); 

		  box825->pack_start(rb_constant);
		  box825->pack_start(rb_rndh);

  //box_dyna->pack_start (*manage(new Gtk::HSeparator()),false, true,0); 
	  Gtk::HBox *box77=manage(new Gtk::HBox(false,10));
   box_dyna->pack_start(*box77);

	  Gtk::HBox *box771=manage(new Gtk::HBox(false,10));
	  //box77->pack_start(*box771,false,false,0);

		para_label=manage( new Gtk::Label ("h=", 0, 0.5));
  	  	box771->pack_start (*para_label , false, true, 0);
  
		Gtk::Adjustment *adjh = manage( new Gtk::Adjustment (0.0, -8.0, 8.0, 0.1, 0.1, 0.0) );

  		spinner_he_val = manage( new Gtk::SpinButton (*adjh, 0, 1) );
  		spinner_he_val->set_wrap (false);
  		spinner_he_val->set_usize (65 , 0);
  		adjh->value_changed.connect(bind(slot(this, &ControlWindow::he_value_cb),spinner_he_val));
  		box771->pack_start (*spinner_he_val, false, true, 0);

	  Gtk::HBox *box772=manage(new Gtk::HBox(false,10));
	  //box77->pack_start(*box772,false,false,0);

	  	Gtk::HBox *box700=manage(new Gtk::HBox(false,0));
	  	box772->pack_start(*manage(new Gtk::Label("digits: ",0)),false,false);
	
	  	Gtk::Adjustment *adjh1=manage(new Gtk::Adjustment(1, 1, 4));
	  	he_digits=manage(new Gtk::HScale(*adjh1)); 
 	  	he_digits->set_digits(0);
	  	adjh1->value_changed.connect(bind(slot(this,&ControlWindow::he_digits_cb),adjh1));
	  	box772->pack_start(*he_digits,true,true);


  Gtk::Frame *frame_prob = manage( new Gtk::Frame ("p") );
  main_vbox->pack_start (*frame_prob, true, true, 10);

	  Gtk::HBox *box11=manage(new Gtk::HBox(false,10));
	  frame_prob->add (*box11);

	  Gtk::HBox *box111=manage(new Gtk::HBox(false,10));
	  box11->pack_start(*box111,false,false,0);

  	  	box111->pack_start (*manage( new Gtk::Label ("p=", 0, prob) )
		, false, true, 0);
  
		Gtk::Adjustment *adjb = manage( new Gtk::Adjustment (prob, 0.0, 1.0, 0.01, 0.1, 0.0) );

  		spinner_p_val = manage( new Gtk::SpinButton (*adjb, 0, 1) );
  		spinner_p_val->set_wrap (false);
    		spinner_p_val->set_digits (2);
  		spinner_p_val->set_usize (65 , 0);
  		adjb->value_changed.connect(bind(slot(this, &ControlWindow::p_value_cb),spinner_p_val));
  		box111->pack_start (*spinner_p_val, false, true, 0);

	  Gtk::HBox *box112=manage(new Gtk::HBox(false,10));
	  box11->pack_start(*box112,false,false,0);

	  	Gtk::HBox *box12=manage(new Gtk::HBox(false,0));
	  	//box112->pack_start(*manage(new Gtk::Label("digits: ",0)),false,false);
	
	  	Gtk::Adjustment *adjb1=manage(new Gtk::Adjustment(2, 1, 4));
	  	p_digits=manage(new Gtk::HScale(*adjb1)); 
 	  	p_digits->set_digits(0);
	  	adjb1->value_changed.connect(bind(slot(this,&ControlWindow::p_digits_cb),adjb1));
	  	box112->pack_start(*p_digits,true,true);

  Gtk::Frame *frame_buttons = manage( new Gtk::Frame ("actions") );
  //main_vbox->pack_start (*frame_buttons, true, true, 10);

  Gtk::HBox *hbox_buttons=manage(new Gtk::HBox(false,0));
  frame_buttons-> add (*hbox_buttons);

  hbox_buttons->pack_start (initer, false, false, 0); 
  hbox_buttons->pack_start (runner, false, false, 0); 
  hbox_buttons->pack_start (restarter, false, false, 0); 
  hbox_buttons->pack_start (stopper, false, false, 0); 
  hbox_buttons->pack_start (eraser, false, false, 0); 
  hbox_buttons->pack_start (button, false, false, 0); 

  set_button_states(true,false,false,false);

  initer.clicked.connect(slot(this, &ControlWindow::init)); 
  runner.clicked.connect(slot(this, &ControlWindow::run)); 
  restarter.clicked.connect(slot(this, &ControlWindow::restart)); 
  stopper.clicked.connect(slot(this, &ControlWindow::stop));

  eraser.clicked.connect(slot(this, &ControlWindow::erase));
  button.clicked.connect(destroy.slot());
  destroy.connect(Gtk::Main::quit.slot());

  Gtk::Frame *frame_threshold = manage( new Gtk::Frame ("threshold") );
  //main_vbox->pack_start (*frame_threshold, true, true, 10);
	  Gtk::VBox *boxd111=manage(new Gtk::VBox(false,0));
	  frame_threshold->add (*boxd111); 
	  Gtk::HBox *boxd222=manage(new Gtk::HBox(false,0));
	  boxd111->pack_start(*boxd222);

	  boxd222->pack_start (threshold_decreaser, false, false, 0); 
	threshold_decreaser.clicked.connect(slot(this, &ControlWindow::decrease_threshold)); 
		
	  boxd222->pack_start (threshold_increaser, false, false, 0); 
	  threshold_increaser.clicked.connect(slot(this, &ControlWindow::increase_threshold)); 
	threshold_label= manage(new Gtk::Label("threshold: ",0));
	  boxd222->pack_start(*threshold_label,false,false);

		
	  adj_threshold.set_value(0.);
	  threshold_value=0.;
	  threshold_wt=manage(new Gtk::HScale(adj_threshold)); 
	  threshold_wt->set_digits(0);
	  adj_threshold.value_changed.connect(bind(slot(this,&ControlWindow::threshold_cb),&adj_threshold));
	 boxd111->pack_start (*manage(new Gtk::HSeparator()),false, true,0); 
	  Gtk::HBox *boxd333=manage(new Gtk::HBox(false,0));
	  boxd111->pack_start (*boxd333, false, false, 0); 
	  boxd333->pack_start(*threshold_wt,true,true);

	  Gtk::HBox *boxd444=manage(new Gtk::HBox(false,0));
	  boxd111->pack_start (*boxd444, false, false, 0); 

	ck_threshold_scroll.set_active(false);
	boxd444->pack_start(ck_threshold_scroll);

	boxd444->pack_start (*manage(new Gtk::VSeparator()),false, true,0); 
	  boxd444->pack_start(*manage(new Gtk::Label(" delay: ",0)),false,false);
	  adj_threshold_scroll_delay.set_value(0);
	  threshold_scroll_delay=0;
	  threshold_scroll_delay_wt=manage(new Gtk::HScale(adj_threshold_scroll_delay)); 
	  threshold_scroll_delay_wt->set_digits(0);
	  adj_threshold_scroll_delay.value_changed.connect(bind(slot(this,&ControlWindow::threshold_scroll_delay_cb),&adj_threshold_scroll_delay));
	  boxd444->pack_start(*threshold_scroll_delay_wt,true,true);

	  update_threshold();


  frame_depth = manage( new Gtk::Frame ("depth") );
  //main_vbox->pack_start (*frame_depth, true, true, 10);

	  Gtk::VBox *box_depth=manage(new Gtk::VBox(false,0));
	  frame_depth-> add (*box_depth);


	     	Gtk::HBox *boxd21=manage(new Gtk::HBox(false,0));
	  	box_depth->pack_start (*boxd21, false, false, 0); 

		  boxd21->pack_start(*manage(new Gtk::Label("ratio: ",0)),false,false);

		  depth_ratio_wt=manage(new Gtk::HScale(adj_ratio)); 
		  depth_ratio_wt->set_digits(0);
		  adj_ratio.value_changed.connect(bind(slot(this,&ControlWindow::depth_ratio_cb),&adj_ratio));
		  boxd21->pack_start(*depth_ratio_wt,true,true);

  		 box_depth->pack_start (*manage(new Gtk::HSeparator()),false, true,0); 
		  Gtk::HBox *boxd22=manage(new Gtk::HBox(false,0));
		  box_depth->pack_start (*boxd22, false, false, 0); 

  	          boxd22->pack_start (depth_decreaser, false, false, 0); 
	  	depth_decreaser.clicked.connect(slot(this, &ControlWindow::decrease_pos)); 
			
  	          boxd22->pack_start (depth_increaser, false, false, 0); 
  		  depth_increaser.clicked.connect(slot(this, &ControlWindow::increase_pos)); 
		visu_label= manage(new Gtk::Label("visu: ",0));
		  boxd22->pack_start(*visu_label,false,false);

			
		  adj_pos.set_value(50.);
		  depth_pos=50.;
		  depth_pos_wt=manage(new Gtk::HScale(adj_pos)); 
		  depth_pos_wt->set_digits(0);
		  adj_pos.value_changed.connect(bind(slot(this,&ControlWindow::depth_pos_cb),&adj_pos));
  		 box_depth->pack_start (*manage(new Gtk::HSeparator()),false, true,0); 
		  Gtk::HBox *boxd33=manage(new Gtk::HBox(false,0));
		  box_depth->pack_start (*boxd33, false, false, 0); 
		  boxd33->pack_start(*depth_pos_wt,true,true);

		  Gtk::HBox *boxd44=manage(new Gtk::HBox(false,0));
		  box_depth->pack_start (*boxd44, false, false, 0); 

	  	ck_depth_scroll.set_active(false);
	  	boxd44->pack_start(ck_depth_scroll);

	  	boxd44->pack_start (*manage(new Gtk::VSeparator()),false, true,0); 
		  boxd44->pack_start(*manage(new Gtk::Label(" delay: ",0)),false,false);
		  adj_depth_scroll_delay.set_value(10);
		  depth_scroll_delay=10;
		  depth_scroll_delay_wt=manage(new Gtk::HScale(adj_depth_scroll_delay)); 
		  depth_scroll_delay_wt->set_digits(0);
		  adj_depth_scroll_delay.value_changed.connect(bind(slot(this,&ControlWindow::depth_scroll_delay_cb),&adj_depth_scroll_delay));
		  boxd44->pack_start(*depth_scroll_delay_wt,true,true);

	          update_pos();

  Gtk::Frame *frame_refresh = manage( new Gtk::Frame ("refresh and display") );
  //main_vbox->pack_start (*frame_refresh, true, true, 10);

	  data_display->connect_refresh_points_synchronized=0;
	  data_display->connect_refresh_points_with_delay=0;
	  data_display->connect_refresh_points_without_delay=1;
	  start_connection();

	  Gtk::VBox *box_refresh=manage(new Gtk::VBox(false,0));
	  frame_refresh-> add (*box_refresh);


	     	Gtk::HBox *box21=manage(new Gtk::HBox(false,0));
	  	box_refresh->pack_start (*box21, false, false, 0); 

		  box21->pack_start(*manage(new Gtk::Label("size: ",0)),false,false);

		  refresh_size_value=100;
		  data_display->refresh_size_update();
		  refresh_size_wt=manage(new Gtk::HScale(adj_size)); 
		  refresh_size_wt->set_digits(0);
		  adj_size.value_changed.connect(bind(slot(this,&ControlWindow::refresh_size_cb),&adj_size));
		  box21->pack_start(*refresh_size_wt,true,true);

		  Gtk::HBox *box22=manage(new Gtk::HBox(false,0));
		  //box_refresh->pack_start (*box22, false, false, 0); 

		  box22->pack_start(*manage(new Gtk::Label("delay: ",0)),false,false);

		  adj_delay.set_value(0);
		  refresh_delay=0;
		  refresh_delay_wt=manage(new Gtk::HScale(adj_delay)); 
		  refresh_delay_wt->set_digits(0);
		  adj_delay.value_changed.connect(bind(slot(this,&ControlWindow::refresh_delay_cb),&adj_delay));
		  box22->pack_start(*refresh_delay_wt,true,true);

	  box_refresh->pack_start (*manage(new Gtk::HSeparator),false, true); 

	  Gtk::HBox *box2_refresh=manage(new Gtk::HBox(false,0));
	  box_refresh->pack_start (*box2_refresh, false, false, 0); 


		  rb_client.set_group(rb_server.group());
		  rb_client.set_active(true);

		  Gtk::HBox *box23=manage(new Gtk::HBox(false,0));
		  box2_refresh->pack_start (*box23, false, false, 0); 

		  box23->pack_start(rb_client);
		  box23->pack_start(rb_server);

	  box2_refresh->pack_start (*manage(new Gtk::VSeparator()),false, true,0); 
		  rb_random.set_group(rb_sweep.group());
		  rb_sweep.set_active(true);

		  Gtk::HBox *box25=manage(new Gtk::HBox(false,0));
	  box2_refresh->pack_start (*box25, false, false, 0); 

		  box25->pack_start(rb_random);
		  box25->pack_start(rb_sweep);

  //box_refresh->pack_start (*manage(new Gtk::HSeparator()),false, true,0); 

	  Gtk::HBox *box3_refresh=manage(new Gtk::HBox(false,0));
	  //box_refresh->pack_start (*box3_refresh, false, false, 0); 

		  rb_noise.set_group(rb_algo.group());
		  rb_noise.set_active(true);

		  Gtk::HBox *box31=manage(new Gtk::HBox(false,0));
	  box3_refresh->pack_start (*box31, false, false, 0); 

		  box31->pack_start(rb_noise);
		  box31->pack_start(rb_algo);

	  box3_refresh->pack_start (*manage(new Gtk::VSeparator()),false, true,0); 
		  ck_auto.set_active(false);

		  Gtk::HBox *box32=manage(new Gtk::HBox(false,0));
	  box3_refresh->pack_start (*box32, false, false, 0); 

		  box32->pack_start(ck_auto);

		  Gtk::HBox *box311=manage(new Gtk::HBox(false,0));
	  box3_refresh->pack_start (*box311, false, false, 0); 

		  ck_bipro.set_active(false);
		  box311->pack_start(ck_bipro);
  ck_bipro.clicked.connect(slot(this,&ControlWindow::algo_cb));

    //hbox_buttons.pack_start (*manage(new Gtk::HSeparator),false, true, 0); 

  //Gtk::Frame *frame_display = manage( new Gtk::Frame ("display") );
  //main_vbox->pack_start (*frame_display, true, true, 10);

    //box_refresh->pack_start (*manage(new Gtk::HSeparator),false, true, 0); 
	  Gtk::HBox *box_display=manage(new Gtk::HBox(false,0));
	  	//box_refresh->pack_start(*box_display);


  Gtk::MenuBar *minus_menubar = manage( new Gtk::MenuBar());
  Gtk::MenuBar *plus_menubar  = manage( new Gtk::MenuBar());

 {
     using namespace Gtk;
     using namespace Menu_Helpers;

     Menu    *minus_menu_colors      = manage( new Menu());
     MenuList& minus_list_file     = minus_menu_colors->items();

	for (int i=0; i<NCOLORS; i++) {
     minus_list_file.push_back(MenuElem(color_names[i], bind(slot(this, &ControlWindow::void_minus_color_int_cb), i)));
	}

     Menu    *plus_menu_colors      = manage( new Menu());
     MenuList& plus_list_file     = plus_menu_colors->items();

	for (int i=0; i<NCOLORS; i++) {
     plus_list_file.push_back(MenuElem(color_names[i], bind(slot(this, &ControlWindow::void_plus_color_int_cb), i)));
	}

     MenuElem black_elem("empty",*minus_menu_colors);
     minus_menubar->items().push_front(black_elem);

     MenuElem plus_elem("occupied",*plus_menu_colors);
     plus_menubar->items().push_front(plus_elem);

   }
   box_display->pack_start (*minus_menubar, false, false);

  minus_color_label = manage( new Gtk::Label ("") );
  minus_color_label->set_text ("is "+black_color_name);
  box_display->pack_start (*minus_color_label, false, true, 5);

  box_display->pack_start (*plus_menubar, false, false);

  plus_color_label = manage( new Gtk::Label ("") );
  plus_color_label->set_text ("is "+white_color_name);
  box_display->pack_start (*plus_color_label, false, true, 5);

  frame_stats = manage( new Gtk::Frame ("statistics") );
  //main_vbox->pack_start (*frame_stats, true, true, 10);

	  Gtk::VBox *vbox_stats=manage(new Gtk::VBox(false,0));
	  frame_stats-> add (*vbox_stats);
	
	  Gtk::HBox *hbox1_stats=manage(new Gtk::HBox(false,0));
	  vbox_stats-> add (*hbox1_stats);
	
	  runtime_label = manage( new Gtk::Label ("") );
  	  hbox1_stats->pack_start (*runtime_label, false, true, 5);

	  refresh_label = manage( new Gtk::Label ("") );
  	  hbox1_stats->pack_start (*refresh_label, false, true, 5);

	  Gtk::HBox *hbox2_stats=manage(new Gtk::HBox(false,0));
	  vbox_stats-> add (*hbox2_stats);
	
	  iter_label = manage( new Gtk::Label ("") );
  	  hbox2_stats->pack_start (*iter_label, false, true, 5);

  Gtk::Main::timeout.connect(slot(this,&ControlWindow::statistics),1000);

  rb_algo.set_active(true);
  data_display->run();

  show_all();

  }

ControlWindow::~ControlWindow()
  {}

int ControlWindow::refresh_points_synchronized() 
{
	return(data_display->refresh_points_synchronized());
}

int ControlWindow::refresh_points_with_delay() 
{
	return(data_display->refresh_points_with_delay());
}

int ControlWindow::refresh_points_without_delay() 
{
	return(data_display->refresh_points_without_delay());
}

int ControlWindow::statistics() 
{
 	gchar buf[64];
	static time_t time1=time(NULL), time2;
	static int count=0;
	static int last_iteration=0;
	static int last_refresh=0;
	static int last_refresh_speed=0;
	static int iter_since_last_size_change=0;
	static int refresh_size_inc=-1;
	int refresh_speed;

	time2=time(NULL);
	float interval=(float)difftime(time2,time1);

    	sprintf (buf, "runtime:%ds", (int)difftime(time2,time_start));
  	runtime_label->set_text (buf);


	int iter=algo.get_iteration();
	if (last_iteration>iter) last_iteration=iter;

	if (interval>10.0) {

    	sprintf (buf, "statistics (%d)", count++);
  	frame_stats->set_label (buf);

    	sprintf (buf, "%d iterations (%.2f/s)", 
		iter, (float)(iter-last_iteration)/interval);
  	iter_label->set_text (buf);
	last_iteration=iter;

	float ratio=
		(float)(hlength*vlength)/(float)(refresh_hsize*refresh_vsize);
refresh_speed=(int)((float)(refresh_number-last_refresh)/interval/ratio);
	last_refresh=refresh_number;
	time1=time2;

	if  (refresh_delay==0) {

    	sprintf (buf, "refresh speed:%d/s", refresh_speed);
  	refresh_label->set_text (buf);
	
	if (ck_auto.get_active() && 
		++iter_since_last_size_change>1) {

	if (refresh_speed<last_refresh_speed) {
		refresh_size_inc=-refresh_size_inc;
	}
	last_refresh_speed=refresh_speed;
	refresh_size_value+=refresh_size_inc;
	iter_since_last_size_change=0;
	if (refresh_size_value,0) {
		refresh_size_value=0;
	}
	if (refresh_size_value>100) {
		refresh_size_value=100;
	}
	adj_size.set_value(refresh_size_value);
  	data_display->refresh_size_update();

	}
	
	} else {
    		sprintf (buf, " ");
  		refresh_label->set_text (buf);
	}

	}
	return (1);
}

void ControlWindow::set_button_states(bool i, bool r, bool t, bool s)
{
	initer.set_sensitive(i);
	runner.set_sensitive(r);
	restarter.set_sensitive(t);
	stopper.set_sensitive(s);
}

void ControlWindow::set_mode_states(bool f, bool w, bool s)
{
		  rb_free.set_active(f);
		  rb_wait.set_active(w);
		  rb_synchro.set_active(s);
}

void ControlWindow::update_prob() {
	algo.set_prob(prob);
}

void ControlWindow::update_temp() {
	algo.set_temp(temp);
}

void ControlWindow::update_field() {
	algo.set_field(field);
}

void ControlWindow::update_threshold() {
	char buf[128];
	data_display->threshold_update();
	algo.set_threshold(threshold);
    	sprintf (buf, "  minimum cluster size=%-3d ", threshold);
	threshold_label->set_text(buf);
}

void ControlWindow::update_depth() {
	data_display->depth_update();
}

void ControlWindow::update_pos() {
	char buf[128];
	data_display->position_update();
	algo.set_pos(position);
    	sprintf (buf, "%d x %d x %d", 
		hlength, vlength, depth);
  	frame_depth->set_label (buf);
    	sprintf (buf, "  visu=%-3d ", position);
	visu_label->set_text(buf);
}

void ControlWindow::T_digits_cb(Gtk::Adjustment *adj)
{
    gfloat val=adj->get_value();

    spinner_T_val->set_digits ((gint)val);
    (spinner_T_val->get_adjustment())->set_step_increment(pow(10, (int)-val));
}

void ControlWindow::he_digits_cb(Gtk::Adjustment *adj)
{
    gfloat val=adj->get_value();

    spinner_he_val->set_digits ((gint)val);
    (spinner_he_val->get_adjustment())->set_step_increment(pow(10, (int)-val));
	if (rb_constant.get_active()) { h_digits=(gint)val; }
	if (rb_rndh.get_active()) { e_digits=(gint)val; }
}

void ControlWindow::p_digits_cb(Gtk::Adjustment *adj)
{
    gfloat val=adj->get_value();

    spinner_p_val->set_digits ((gint)val);
    (spinner_p_val->get_adjustment())->set_step_increment(pow(10, (int)-val));
}

void ControlWindow::depth_ratio_cb(Gtk::Adjustment *adj)
{
    depth_ratio_value=adj->get_value();
    set_button_states(true, false, true, true); 
}

void ControlWindow::refresh_size_cb(Gtk::Adjustment *adj)
{
    refresh_size_value=adj->get_value();
    data_display->refresh_size_update();
}

void ControlWindow::start_connection() {
    	if (data_display->connect_refresh_points_synchronized==1) {
connection=Gtk::Main::idle.connect(SigC::slot(this,&ControlWindow::refresh_points_synchronized)); }
    	if (data_display->connect_refresh_points_with_delay==1) {
connection=Gtk::Main::timeout.connect(SigC::slot(this,&ControlWindow::refresh_points_with_delay), refresh_delay*500); }
    	if (data_display->connect_refresh_points_without_delay==1) {
connection=Gtk::Main::idle.connect(SigC::slot(this,&ControlWindow::refresh_points_without_delay)); }
	data_display->set_connected_true();
}

void ControlWindow::end_connection() {
    connection.disconnect(); 
	data_display->set_connected_false();
}

void ControlWindow::algo_speed_cb(Gtk::Adjustment *adj)
{
    gfloat val=adj->get_value();

    algo_speed=(int)val;
	algo.set_speed(algo_speed);
    
}

void ControlWindow::threshold_scroll_delay_cb(Gtk::Adjustment *adj)
{
    gfloat val=adj->get_value();

    threshold_scroll_delay=(int)val;
    
}

void ControlWindow::depth_scroll_delay_cb(Gtk::Adjustment *adj)
{
    gfloat val=adj->get_value();

    depth_scroll_delay=(int)val;
    
}


void ControlWindow::refresh_delay_cb(Gtk::Adjustment *adj)
{
    gfloat val=adj->get_value();

    refresh_delay=(int)val;
    
    if (refresh_delay>0) {
	/* the delay is not well handled with timeout, hence I 
	implement another mechanism inside refresh_points...*/
    	data_display->connect_refresh_points_with_delay=1;
    	data_display->connect_refresh_points_without_delay=0;
    	data_display->connect_refresh_points_synchronized=0;
    } else {
    	data_display->connect_refresh_points_without_delay=1;
    	data_display->connect_refresh_points_with_delay=0;
    	data_display->connect_refresh_points_synchronized=0;
    }
    start_connection();
}

void ControlWindow::increase_threshold() {
    if (threshold<1000){
    threshold_value=(float)(1000.*(threshold+1.2)/(float)
	(hlength*vlength*depth));
    } else {
    threshold_value+=1.;
    /*threshold_value=(float)(1000.*(threshold+ (hlength*vlength*depth)/10000)
		/(float) (hlength*vlength*depth));*/
	}
    if (threshold_value>100) { threshold_value=100; }
    adj_threshold.set_value(threshold_value);
    update_threshold();
}

void ControlWindow::decrease_threshold() {
    if (threshold<1000){
    threshold_value=(float)(1000.*(threshold-0.9)/(float)
	(hlength*vlength*depth));
    } else {
    /*threshold_value=(float)(1000.*(threshold-(hlength*vlength*depth)/10000)
		/(float) (hlength*vlength*depth));*/
    threshold_value-=1.;
	}
    if (threshold_value<0) { threshold_value=0; }
    adj_threshold.set_value(threshold_value);
    update_threshold();
}

void ControlWindow::threshold_cb(Gtk::Adjustment *adj)
{
    gfloat val=adj->get_value();

    threshold_value=val;
    update_threshold();
}

void ControlWindow::increase_pos() {
    depth_pos=(float)(100.*(position+1.001)/(float)(depth-1));
    if (depth_pos>100) { depth_pos=100; }
    adj_pos.set_value(depth_pos);
    update_pos();
}

void ControlWindow::decrease_pos() {
    depth_pos=(float)(100.*(position-0.999)/(float)(depth-1));
    if (depth_pos<0) { depth_pos=0; }
    adj_pos.set_value(depth_pos);
    update_pos();
}

void ControlWindow::depth_pos_cb(Gtk::Adjustment *adj)
{
    gfloat val=adj->get_value();

    depth_pos=val;
    update_pos();
}

/***********************************************************************************
   DataDisplay
************************************************************************************/

void DataDisplay::set_connected_false() {
	connected=false;
	lattice_window.drawing_area.set_connected_true();
}

void DataDisplay::set_connected_true() {
	connected=true;
	lattice_window.drawing_area.set_connected_true();
}

void DataDisplay::start_connection() {
	if (!connected && control_window_ready) 
		control_window->start_connection();
}

void DataDisplay::end_connection() {
	if (connected) 
		control_window->end_connection();
}

DataDisplay::DataDisplay() : lattice_window(this) {
  mtrand.seed( time(NULL));
  connected=false;
  control_window_ready=false;
}

DataDisplay::~DataDisplay() {
}

int DataDisplay::refresh_points_synchronized() {
    GdkRectangle update_rect;

	refresh_number++;

	int xmin, xmax, ymin, ymax;

	xmin=hlength; xmax=0;
	ymin=vlength; ymax=0;

	int x=0, y=0;
	int width, height;

	width=hlength;
	height=vlength;

	update_rect.x = (int)x;
	update_rect.y = (int)y;
	update_rect.width = width;
	update_rect.height = height;

	//algo.wait_and_lock_changed_sites();

	if (control_window->rb_algo.get_active() &&
 		control_window->rb_client.get_active() &&
			control_window->rb_sites.get_active()) {

	int i,j;

	while (algo.next_site(&i, &j)) {
	if (xmin>i) xmin=i;
	if (xmax<i) xmax=i;
	if (ymin>j) ymin=j;
	if (ymax<j) ymax=j;
	if (algo.label_status(i,j)>0) {
(lattice_window.drawing_area).put_white_pixel(i,j); }
	else {
(lattice_window.drawing_area).put_black_pixel(i,j); }

	} }

	if (control_window->rb_algo.get_active() &&
 		control_window->rb_client.get_active() &&
			control_window->rb_labels.get_active()) {

	int i,j;

	while (algo.next_site(&i, &j)) {
	if (xmin>i) xmin=i;
	if (xmax<i) xmax=i;
	if (ymin>j) ymin=j;
	if (ymax<j) ymax=j;
(lattice_window.drawing_area).put_pixel(i,j, 
		algo.label_status(i,j) % ncolors);

	} }
	
	if (control_window->rb_algo.get_active() &&
 		control_window->rb_server.get_active() &&
			control_window->rb_sites.get_active()) {

	int i,j;
	while (algo.next_site(&i, &j)) {
	if (xmin>i) xmin=i;
	if (xmax<i) xmax=i;
	if (ymin>j) ymin=j;
	if (ymax<j) ymax=j;
	if (algo.label_status(i,j)>0) {
		(lattice_window.drawing_area).draw_white_point(i,j);
	} else { (lattice_window.drawing_area).draw_black_point(i,j); }  

	} }

	if (control_window->rb_algo.get_active() &&
 		control_window->rb_server.get_active() &&
			control_window->rb_labels.get_active()) {

	int i,j;
	while (algo.next_site(&i, &j)) {
	if (xmin>i) xmin=i;
	if (xmax<i) xmax=i;
	if (ymin>j) ymin=j;
	if (ymax<j) ymax=j;
(lattice_window.drawing_area).draw_point(i,j, 
		algo.label_status(i,j) % ncolors);
	} }

	if (xmin<xmax && ymin<ymax) {

	update_rect.x = (int)xmin;
	update_rect.y = (int)ymin;
	update_rect.width = xmax-xmin;
	update_rect.height = ymax-ymin;

	if (control_window->rb_client.get_active()) {
		(lattice_window.drawing_area).draw_image(xmin,ymin,xmax-xmin,
	ymax-ymin); }
	(lattice_window.drawing_area).draw(&update_rect);

	}
	
	//algo.unlock_changed_sites();

	return connect_refresh_points_synchronized;
}

int DataDisplay::refresh_points_without_delay() {
    GdkRectangle update_rect;

	refresh_number++;

	static int x=0, y=0;
	static int width=0, height=0;

	static int threshold_scroll_iteration=0;
	static int threshold_scroll_inc=-1;
	
    	if (control_window->ck_threshold_scroll.get_active()) { 
		if (threshold_scroll_iteration++>threshold_scroll_delay) {
			threshold_scroll_iteration=0; 
	if (threshold_scroll_inc>0) control_window->increase_threshold();
		else control_window->decrease_threshold();
		if (threshold==0) { threshold_scroll_inc=+1; }
		else if (threshold==hlength*vlength*depth/10)
			{ threshold_scroll_inc=-1; }
		}
	}

	static int depth_scroll_iteration=0;
	static int depth_scroll_inc=-1;
	
    	if (control_window->ck_depth_scroll.get_active()) { 
		if (depth_scroll_iteration++>depth_scroll_delay) {
			depth_scroll_iteration=0; 
		if (depth_scroll_inc>0) control_window->increase_pos();
		else control_window->decrease_pos();
		if (position==0) { depth_scroll_inc=+1; }
		else if (position==depth-1) { depth_scroll_inc=-1; }
		}
	}

	if (control_window->rb_random.get_active()) {
		x= mtrand.randInt(hlength-refresh_hsize);
		y= mtrand.randInt(vlength-refresh_vsize);
		width=refresh_hsize; height=refresh_vsize;
	} else if (control_window->rb_sweep.get_active()) {
		if (x+width<hlength) {
			x+=refresh_hsize;
			if (x+refresh_hsize>hlength) width=hlength-x;
			else width=refresh_hsize;
		} else { x=0; width=refresh_hsize; 
			if (y+height<vlength) {
				y+=refresh_vsize;
				if (y+refresh_vsize>vlength) height=vlength-y;
				else height=refresh_vsize;
			} else {
				y=0; height=refresh_vsize;	
			}
		}
/* this is to take care of what happens in case of a resize:
	the old values of y and height might be incompatible!! */
		if (y+height>vlength) { y=0; height=refresh_vsize; }
		}
	update_rect.x = (int)x;
	update_rect.y = (int)y;
	update_rect.width = width;
	update_rect.height = height;

	if (control_window->rb_noise.get_active() &&
 		control_window->rb_client.get_active() &&
			control_window->rb_sites.get_active()) {

	for (int i=x; i<x+width; i++) {
		for (int j=y; j<y+height; j++) {
			if (mtrand.rand()<prob) {
		(lattice_window.drawing_area).put_black_pixel(i,j);
		} else { (lattice_window.drawing_area).put_white_pixel(i,j); }  
	} } } 


	if (control_window->rb_noise.get_active() &&
 		control_window->rb_client.get_active() &&
			control_window->rb_labels.get_active()) {

	for (int i=x; i<x+width; i++) {
		for (int j=y; j<y+height; j++) {
(lattice_window.drawing_area).put_pixel(i,j, mtrand.randInt(ncolors));

	} } }

	if (control_window->rb_noise.get_active() &&
 		control_window->rb_server.get_active() &&
			control_window->rb_sites.get_active()) {

	for (int i=x; i<x+width; i++) {
		for (int j=y; j<y+height; j++) {
			if (mtrand.rand()<prob) {
				(lattice_window.drawing_area).draw_black_point(i,j);
			} else { (lattice_window.drawing_area).draw_white_point(i,j); }  

	} } }

	if (control_window->rb_noise.get_active() &&
 		control_window->rb_server.get_active() &&
			control_window->rb_labels.get_active()) {

	for (int i=x; i<x+width; i++) {
		for (int j=y; j<y+height; j++) {
(lattice_window.drawing_area).draw_point(i,j, mtrand.randInt(ncolors));
	} } }	
	
	if (control_window->rb_algo.get_active() &&
 		control_window->rb_client.get_active() &&
			control_window->rb_sites.get_active()) {

	for (int i=x; i<x+width; i++) {
		for (int j=y; j<y+height; j++) {
			if (algo.label_status(i,j)>0) {
				(lattice_window.drawing_area).put_white_pixel(i,j);
			} else { (lattice_window.drawing_area).put_black_pixel(i,j); }  

	} } }


	if (control_window->rb_algo.get_active() &&
 		control_window->rb_client.get_active() &&
			control_window->rb_labels.get_active()) {

	for (int i=x; i<x+width; i++) {
		for (int j=y; j<y+height; j++) {
(lattice_window.drawing_area).put_pixel(i,j, 
		algo.label_status(i,j) % ncolors);
	} } }
	
	if (control_window->rb_algo.get_active() &&
 		control_window->rb_server.get_active() &&
			control_window->rb_sites.get_active()) {

	for (int i=x; i<x+width; i++) {
		for (int j=y; j<y+height; j++) {
			if (algo.label_status(i,j)>0) {
				(lattice_window.drawing_area).draw_white_point(i,j);
			} else { (lattice_window.drawing_area).draw_black_point(i,j); }  

	} } }

	if (control_window->rb_algo.get_active() &&
 		control_window->rb_server.get_active() &&
			control_window->rb_labels.get_active()) {

	for (int i=x; i<x+width; i++) {
		for (int j=y; j<y+height; j++) {
(lattice_window.drawing_area).draw_point(i,j, 
		algo.label_status(i,j) % ncolors);
	} } } 

	if (control_window->rb_client.get_active()) {
		(lattice_window.drawing_area).draw_image(x,y,width,height); }
	(lattice_window.drawing_area).draw(&update_rect);

	return connect_refresh_points_without_delay;
}


int DataDisplay::refresh_points_with_delay() 
{
    GdkRectangle update_rect;

	static time_t time1=time(NULL), time2;

	time2=time(NULL);
	float interval=(float)difftime(time2,time1);

	if (interval<refresh_delay) { 
		return connect_refresh_points_with_delay;
	}

	time1=time2;

    refresh_number++;

    int x= mtrand.randInt(hlength-refresh_hsize);
    int y= mtrand.randInt(vlength-refresh_vsize);
    update_rect.x = (int)x;
    update_rect.y = (int)y;
    update_rect.width = refresh_hsize;
    update_rect.height = refresh_vsize;
	if (control_window->rb_noise.get_active()) {

	if (control_window->rb_client.get_active()) {

	for (int i=x; i<x+refresh_hsize; i++) {
		for (int j=y; j<y+refresh_vsize; j++) {
			if (mtrand.rand()<prob) {
				(lattice_window.drawing_area).put_black_pixel(i,j);
			} else { (lattice_window.drawing_area).put_white_pixel(i,j); }  
	}}

	} else if (control_window->rb_server.get_active()) {

	for (int i=x; i<x+refresh_hsize; i++) {
		for (int j=y; j<y+refresh_vsize; j++) {
			if (mtrand.rand()<prob) {
				(lattice_window.drawing_area).draw_black_point(i,j);
			} else { (lattice_window.drawing_area).draw_white_point(i,j); }  

	} }
	
	} 

	} else if (control_window->rb_algo.get_active()) {

	if (control_window->rb_client.get_active()) {

	for (int i=x; i<x+refresh_hsize; i++) {
		for (int j=y; j<y+refresh_vsize; j++) {
			if (algo.label_status(i,j)>0) {
				(lattice_window.drawing_area).put_white_pixel(i,j);
			} else { (lattice_window.drawing_area).put_black_pixel(i,j); }  

	} }

	} else if (control_window->rb_server.get_active()) {

	for (int i=x; i<x+refresh_hsize; i++) {
		for (int j=y; j<y+refresh_vsize; j++) {
			if (algo.label_status(i,j)>0) {
				(lattice_window.drawing_area).draw_white_point(i,j);
			} else { (lattice_window.drawing_area).draw_black_point(i,j); }  

	} }

	} } 

	if (control_window->rb_client.get_active()) {
		(lattice_window.drawing_area).draw_image(x,y,refresh_hsize,refresh_vsize); }
		(lattice_window.drawing_area).draw(&update_rect);

	return connect_refresh_points_with_delay;
}

void DataDisplay::connect_synchronized() {
}

void DataDisplay::connect_with_delay() {
//Gtk::Main::timeout.connect(SigC::slot(this,&DataDisplay::refresh_points_with_delay), 
//refresh_delay*500); 
}

void DataDisplay::connect_without_delay() {
//Gtk::Main::idle.connect(SigC::slot(this,&DataDisplay::refresh_points_without_delay));
//Gtk::Main::idle.connect(slot(this,&DataDisplay::stop));
}

void DataDisplay::init() {
	time_start=time(NULL);
	if (control_window_ready) {
	control_window->set_button_states(true,true,false,false); 
	control_window->update_depth();
	control_window->update_pos();
	}
	//algo.init(hlength, vlength, depth, prob, temp, field, position);
algo.init(hlength, vlength, depth, prob, temp, 
		position, threshold, algo_speed, synchronize, wait);
}

void DataDisplay::run() {
    if (control_window_ready) {
    algo_t al=Static;
    if (control_window->rb_static.get_active()) { 
	          al=Static;
		  control_window->set_mode_states(false, true, false);
	 }
    //else if (control_window->rb_threshold.get_active()) { al=Threshold; }
    else if (control_window->rb_dynamic.get_active()) { 
		al=Dynamic; 
		control_window->set_mode_states(false, true, false);
	}
    int num=1;
    if (control_window->ck_bipro.get_active()) { num=2; }
    else { num=1; }
    launch_computation_thread(al,num);
    control_window->set_button_states(false,false,true,true);
    control_window->statistics();
    }
}

void DataDisplay::restart() {
	stop();
	init();
	run();
}

void DataDisplay::stop() {
    if (nthreads>0) { stop_computation_threads(); 
    	control_window->set_button_states(true,true,false,false);}
}

void DataDisplay::erase() {
	lattice_window.drawing_area.erase();
}

void DataDisplay::show() {
	lattice_window.show();
}

void DataDisplay::set_black_color() {
	lattice_window.drawing_area.set_black_color(); 
}

void DataDisplay::set_white_color() {
	lattice_window.drawing_area.set_white_color(); 
}

void DataDisplay::refresh_size_update()
{
    if (refresh_size_value<30) { 
	refresh_hsize= (int)refresh_size_value;
	refresh_vsize= (int)refresh_size_value;
    } else if (refresh_size_value>70) {
	refresh_hsize= hlength-100+(int)refresh_size_value;
	refresh_vsize= vlength-100+(int)refresh_size_value;
    } else  {
	    refresh_hsize=30+((int)refresh_size_value-30)*(hlength-60) / 40;
	    refresh_vsize=30+((int)refresh_size_value-30)*(vlength-60) / 40;
    }
}

/* 1 unit of the threshold bar corresponds to the number of pixels 
	divided by 1000 */
void DataDisplay::threshold_update() {
	threshold=(int)(hlength*vlength*depth*threshold_value/1000);
}

void DataDisplay::position_update() {
	position=(int)((depth-1)*depth_pos/100);
}

void DataDisplay::depth_update() {
    if (depth_ratio_value<30) { 
    	depth=(int)depth_ratio_value;
    } else  {
    	depth=(int)(hlength*depth_ratio_value/100);
    }
}

}

int main (int argc, char *argv[])
{
	cout << "

    Gnu-Percolation is a program to simulate 2D percolation.
    Copyright (C) 2001 Raphael Cerf

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    E-mail contact: Raphael.Cerf@math.u-psud.fr
    Ordinary mail: Raphael Cerf, Mathematique, batiment 425,
	Universite Paris Sud, 91405 Orsay Cedex

	";
	
  Gtk::Main app(argc,argv);
  interface::DataDisplay data_display;
  interface::ControlWindow control_window(&data_display);

  app.run();
  return 0;
}

